// Program..............NETLOCK.PRG
//
// Application..........GENERAL PURPOSE
//
// Author...............LARRY DYSERT
// Copyright............RELEASED TO PUBLIC DOMAIN
//
// Clipper Version .....5.2
//
// Notes................Replacement for 
//    RecLock(),FilLock(),and AddRec()
//

// required header files
*#include "common.ch"  // or COMMAND.CH of 5.2?
#include "Fivewin.ch"
**#include "FWPlus.ch"
#include "mylib.ch"

// manifest constants
#define RETRY_MSG     "Continue Attempt to Lock Record or File?"

FUNCTION NetLock( nType,lReleaseLocks,nSeconds,cAlias,cDel ) 

   local nWaitTime
   local lContinue := .t.
   local lSuccess  := .f.
   local bOperation
   local xIdentifier
   // nType must be passed and valid
   if .not. ( valtype( nType ) == "N" ) .or. ;
     ( ( .not. ( nType == 1 ) ) .and.  ;
       ( .not. ( nType == 2 ) ) .and.  ;
       ( .not. ( nType == 3 ) ) )

      // post alert - early exit
      MsgAlert( ;
      "Invalid Argument Passed to NetLock", )
      return lSuccess
   endif

   // provide defaults to passed parameters
  DEFAULT lReleaseLocks TO .f.
   // maintain existing locks

  DEFAULT nSeconds      TO 5
   // standard retry time

  DEFAULT cAlias        TO " "

  DEFAULT cDel          TO ""

   // initialize nWaitTime
   nWaitTime := nSeconds

   // initialize bOperation
   // (recordlock/filelock/append)
   // and appropriate xIdentifier
   do case
      case nType == 1
         // record lock
         xIdentifier :=  ;
         iif( lReleaseLocks, Nil, recno() )
*         bOperation  := { |x| dbrlock( x ) }    // C 5.2
         bOperation  := { |x| rlock() }
      case nType == 2
         // file lock
         bOperation := { |x| flock() }
      case nType == 3 .AND. EMPTY(cDel)
         // append blank
         xIdentifier := lReleaseLocks
         bOperation  := ;
         { |x| dbAppend( x ), .not. neterr() }
      case nType == 3 .AND. !EMPTY(cDel)
         // reuse an old record
         RecycleRec(cDel)
         xIdentifier :=  ;
         iif( lReleaseLocks, Nil, recno() )
*         bOperation  := { |x| dbrlock( x ) }    // C 5.2
         bOperation  := { |x| rlock() }
   endcase

   // while continuing to lock record or file
   while lContinue
      CursorWait()
      // while .not. timed-out
      while nSeconds > 0 .and. lContinue
         // attempt appropriate lock
         if eval( bOperation, xIdentifier )
            nSeconds  := 0
            lSuccess  := .t.
            lContinue := .f.
         else
            // if lock failed
            // use tone() to delay 1/2 second
            tone( 1, 9 )
            nSeconds := nSeconds - .5
         endif
      enddo

      if .not. lSuccess
         nSeconds  := nWaitTime
         lContinue := YesNo( RETRY_MSG,cAlias )
      endif
   enddo
   SysRefresh()

return lSuccess

//
// eof NetLock()
//

// simple example of a support function to obtain
// user input on whether to keep retrying to
// open the file

STATIC FUNCTION YesNo( cMessage,cAlias )
   local cYesNo  := .f.,nLen

  IF EMPTY(cAlias)
     ELSE
      cMessage := "Continue Attempt to Lock " + cAlias + " File?"
  ENDIF

   nLen    := len( cMessage )

   // display box and message
cYesNo := MsgYesNo( cMessage, "Please select:" )


return ( cYesNo == .t. )
//
//
// eof YesNo()
//

PROCEDURE Nonlock(cFilenam)
MsgAlert(cFilenam + " lock could not be obtained", "Alert")

RETURN
*------------------------------
STATIC FUNCTION RecycleRec(cDel)

   LOCAL lSuccess       := .f.
*   LOCAL nIter          := 1
   LOCAL cTag           := ""
   LOCAL cAlias      := alias()
   

   DEFAULT cDel TO ""

   cTag := ordName()

   CursorWait()
   // cAlias := upper(alltrim(cAlias))

   set deleted off

   IF ordNumber(cDel) = 0
      INDEX ON &cDel TAG &cDel
   ENDIF

   IF !EMPTY(cDel) .AND. ordNumber(cDel) > 0
      SET ORDER TO TAG &cDel
      dbSeek("+")
     ELSE
      dbgobottom()
   ENDIF

   IF deleted() .OR. (cAlias)->&cDel = "+"

      IF (ALIAS())->( NetLock2( RECORDLOCK ) )
         dbrecall()
         lSuccess := .t.
        ELSE
         NONLOCK(ALIAS())
         RETURN .f.
      ENDIF
*cc
*msginfo(RECNO(),"RECNO() - recycle")

     ELSE  // No recycled records available, so...

*      for nIter := 1 to 10  // Make 10 blank records available for future use.
        IF (ALIAS())->( NetLock2( APPENDREC ) ) //dbappend()

*           IF RecLock( LOCK_SECONDS ) // HST orig. code
            BlankRec(cDel)             // Blank all fields
*           IF nIter <> 10             // Unlock all except that last!
*              dbdelete()
*              dbunlock()
*           ENDIF
            lSuccess := .t.
          ELSE
            NONLOCK(ALIAS())
            RETURN .f.
         ENDIF
*         SysRefresh()
*         CursorWait()
*      next

   ENDIF
  
   IF !EMPTY(cDel)
      IF (ALIAS())->( NetLock2( RECORDLOCK ) )
        ELSE
         NONLOCK(ALIAS())
         RETURN .f.
      ENDIF
      ( cAlias )->&cDel := " "
      dbRecall()   // Just to make sure!
   ENDIF

   IF !EMPTY(cTag)
      SET ORDER TO TAG &cTag
   ENDIF
   ( cAlias )->( dbCommit() )
   ( cAlias )->( dbUnLock() )

   CursorArrow()

   set deleted on

RETURN( lSuccess )
*------------------------

FUNCTION BlankRec(cDel)
   LOCAL nFields     := fcount()
   LOCAL cField      := ""
   LOCAL cType       := ""
   LOCAL cAlias      := alias()
   LOCAL nFieldNo

   DEFAULT cDel TO ""

   IF (ALIAS())->( NetLock2( RECORDLOCK ) )
         DELETE
*         lSuccess := .t.
        ELSE
         NONLOCK(ALIAS())
         RETURN nil  //.f.
   ENDIF

   for nFieldNo := 1 to nFields
      cField := ( cAlias )->( fieldname(nFieldNo)  )
      cType  := type( ((cAlias)->( cField )) )
      do case
         CASE cType == "C"
            FIELD->&cField   := chr(254)
         CASE cType == "D"
            FIELD->&cField   := ctod("")
         CASE cType == "L"
            FIELD->&cField   := .f.
         CASE cType == "M"
            FIELD->&cField   := ""
         CASE cType == "N"
            FIELD->&cField   := 0
      endcase
      SysRefresh()
      CursorWait()
   next

   IF !EMPTY(cDel)
      FIELD->&cDel := "+"
   ENDIF

   ( cAlias )->( dbCommit() )
   ( cAlias )->( dbUnLock() )

RETURN( NIL )
*------------------------------
  // 2nd version called only from RecycleRec() and BlankRec() - used to prevent recursion...
STATIC FUNCTION NetLock2( nType,lReleaseLocks,nSeconds,cAlias ) 

   local nWaitTime
   local lContinue := .t.
   local lSuccess  := .f.
   local bOperation
   local xIdentifier

   // nType must be passed and valid
   if .not. ( valtype( nType ) == "N" ) .or. ;
     ( ( .not. ( nType == 1 ) ) .and.  ;
       ( .not. ( nType == 2 ) ) .and.  ;
       ( .not. ( nType == 3 ) ) )

      // post alert - early exit
      MsgAlert( ;
      "Invalid Argument Passed to NetLock", )
      return lSuccess
   endif

   // provide defaults to passed parameters
  DEFAULT lReleaseLocks TO .f.
   // maintain existing locks

  DEFAULT nSeconds      TO 5
   // standard retry time

  DEFAULT cAlias        TO " "

   // initialize nWaitTime
   nWaitTime := nSeconds

   // initialize bOperation
   // (recordlock/filelock/append)
   // and appropriate xIdentifier
   do case
      case nType == 1
         // record lock
         xIdentifier :=  ;
         iif( lReleaseLocks, Nil, recno() )
*         bOperation  := { |x| dbrlock( x ) }    // C 5.2
         bOperation  := { |x| rlock() }
      case nType == 2
         // file lock
         bOperation := { |x| flock() }
      case nType == 3
         // append blank
         xIdentifier := lReleaseLocks
         bOperation  := ;
         { |x| dbAppend( x ), .not. neterr() }
   endcase

   // while continuing to lock record or file
   while lContinue
      CursorWait()
      // while .not. timed-out
      while nSeconds > 0 .and. lContinue
         // attempt appropriate lock
         if eval( bOperation, xIdentifier )
            nSeconds  := 0
            lSuccess  := .t.
            lContinue := .f.
         else
            // if lock failed
            // use tone() to delay 1/2 second
            tone( 1, 9 )
            nSeconds := nSeconds - .5
         endif
      enddo

      if .not. lSuccess
         nSeconds  := nWaitTime
         lContinue := YesNo( RETRY_MSG,cAlias )
      endif
   enddo
   SysRefresh()

return lSuccess

// end of program - NETLOCK.PRG