// Program..............NETOPEN.PRG
//
// Application..........GENERAL PURPOSE
//
// Author...............LARRY DYSERT
// Copyright............RELEASED TO PUBLIC DOMAIN
//
// Clipper Version .....5.2
//
// Notes................Replacement for NetUse()
//
// required header files
*#include "common.ch"  // or COMMAND.CH of 5.2?
#include "Fivewin.ch"
**#include "FWPlus.ch"
#include "mylib.ch"
**#include "box.ch"
**#include "inkey.ch"

// manifest constants
#define RETRY_MSG     "Continue Attempt to Open Database?"
**#define YESNO_COLOR   "R/W"

FUNCTION NetOpen( cFile, lMode, nSeconds, cAlias, lNewArea, cDriver, lReadOnly)

   local nWaitTime
   local lContinue := .t.
   local lSuccess  := .f., cOldDriver

   // provide defaults to passed parameters

   DEFAULT lMode      TO .f.        // shared mode

   DEFAULT nSeconds   TO 5          // standard retry time

   DEFAULT cAlias     TO cFile      // same as database name

   DEFAULT lNewArea   TO .t.        // open in unused workarea

*   IF EMPTY(cDriver) ; cDriver := "ADS" ; ENDIF  
   IF EMPTY(cDriver) ; cDriver := "DBFCDX" ; ENDIF 
*   DEFAULT cDriver    TO "ADS"   // standard driver
   DEFAULT cDriver    TO "DBFCDX"   // standard driver
**   DEFAULT cDriver    TO "DBFNTX"   // standard driver

   DEFAULT lReadOnly  TO .f.        // open read/write

    // Verify file name given...
    IF EMPTY(cFile)
       MsgAlert("No file name given for file opening. Will cancel.")
       RETURN .f.
    ENDIF

    IF EMPTY(cAlias)
       cAlias := ""
    ENDIF

    IF AT("\",cAlias) > 0
       cAlias := SUBSTR(cAlias,RAT("\",cAlias)+1,LEN(cAlias))
    ENDIF

    cFile := ALLTRIM(cFile)

    // initialize nWaitTime
    nWaitTime := nSeconds

    // verify file exists
    if .not. file( cFile + ".DBF" )
       MsgAlert(cFile+".DBF file not in this directory.","Netopen error:")
       lContinue := .f.
    endif

    // verify driver is valid
*    if ascan( RddList(), cDriver ) == 0    // RddList() not avail in 5.01a
*       lContinue := .f.
*    endif

   IF cAlias == "User"
      nSeconds := 1
   ENDIF


   IF SELECT(cAlias) = 0
    // while continuing to attempt open
    while lContinue
       CursorWait()
       // while .not. timed-out
       while nSeconds > 0 .and. lContinue
         dbUseArea( lNewArea, cDriver, cFile, cAlias, (.not. lMode), lReadOnly)

         // check for success/failure
         if neterr()
             // if open failed
             // use tone() to delay 1 second
             tone( 1, 18 )
             nSeconds--
          else
             // if open successful
             nSeconds  := 0
             lSuccess  := .t.
             lContinue := .f.
         endif
       enddo   

       if .not. lSuccess
          nSeconds  := nWaitTime
          IF cAlias == "User"
              lContinue := .f.
              MsgAlert("All users must sign off before reset.")
            ELSE
              lContinue := YesNo( "Continue attempt to open "+cAlias+" file?" )
          ENDIF
       endif
    enddo
    SysRefresh()
   ELSE
     lSuccess := .t.
     SELECT (cAlias)

/*   // Warning not needed if index selection is always made AFTER each call
     // to open or select a file. Also, this message means nothing to user...
     IF UPPER(ALLTRIM(cAlias)) # "ROSTER"
        MsgAlert("File "+cAlias+" already open, but"+CRLF+;
                 "correct index may not be in use.")
     ENDIF
*/

  ENDIF      // If SELECT(cAlias) = 0

return lSuccess
//
// eof NetOpen()
//

// simple example of a support function to obtain
// user input on whether to keep retrying to
// open the file

STATIC FUNCTION YesNo( cMessage )
   local cYesNo  := .f.
   local nLen    := len( cMessage )

*   local nTop    := maxrow() - 5
*   local nLeft   := int( maxcol() - 5 - nLen ) / 2
*   local nRight  := maxcol() - nLeft
*   local nBottom := maxrow() - 3

*   local cScrn   :=  ;
*       saveScreen( nTop, nLeft, nBottom, nRight )

   // display box and message

cYesNo := MsgYesNo( cMessage,"Please select:" )
*   dispbox( nTop, nLeft, nBottom, nRight, B_DOUBLE + " ", YESNO_COLOR )
*   @ nTop + 1, nLeft + 3 say cMessage  color YESNO_COLOR

   // loop until valid keypress or ESC pressed
*   while .not. cYesNo $ "YN"
*      cYesNo := upper( chr( inkey( 0 ) ) )
*      if lastkey() == K_ESC
*         cYesNo := "N"
*      endif
*   enddo

*  restScreen( nTop, nLeft, nBottom, nRight, cScrn )

return ( cYesNo == .t. )
//
//
// eof YesNo()
//
PROCEDURE Nonopen(cFilenam)
MsgAlert(cFilenam +" File is not available","Alert")

*     cMsgscr := SAVESCREEN(14,20,21,58)
*     @ 14,20 CLEAR TO 21,58
*     @ 14,20,21,58 BOX B_DOUBLE_SINGLE
*     ?? CHR(7)
*     @ 16,25 SAY cFilenam +" File is not available"
*     @ 18,25 SAY "   Press ENTER to continue"
*     INKEY(0) 
*     @ 14,20 CLEAR TO 21,58
*     RESTSCREEN(14,20,21,58,cMsgscr)
RETURN

// end of program - NETOPEN.PRG