*------------------------------------------------------------------------
* PlusTest.PRG                                  R.W. Seiler  Jan 18, 2008
*  Functional samples and tests using Protection PLUS ver. 4.4.1
*  for use with: FiveWin for Harbour (FWH) and xHarbour
*
*  If using a Gui other than FWH, then Main() and BuildMenu() will need
*  to be changed.
*
*  IMPORTANT: see functions GetAppStats() and the Exit Proc Endall() for
*             code that MUST be included in your app's main prg in order
*             to implement PLUS.
*
*  ALSO: must search/replace LF filenames here: MyApp1, MyApp2, MyApp3
*        - replace with your actual LF file names.
*------------------------------------------------------------------------
/*

Functions and procedures in this prg:

  // The app's primary function...
FUNCTION Main(cVer)

  // Create the app's main menu system...
STATIC FUNCTION BuildMenu()

  // Process steps and tests required upon launching the app...
PROCEDURE Startup()

  // Reset the values in the License File to the initial defaults...
PROCEDURE Reset(nVersion)

// Pass some static global values to ProtPlus() on call...
FUNCTION GetAppStats()

  // Mandate app exit...
PROCEDURE Exit()

  // The procedure to be performed upon every exit of the app...
EXIT PROCEDURE EndAll()

  // XBase++ requirement
PROCEDURE AppSys  

*/
#include "FiveWin.ch"
#include "KeyLib.ch"
*#include "KeyLibB.ch" // For temp use instead of Keylib.ch to Clipper
                      // test-compile in order to easily see /w warning errors. 
                      // - because Keylib.ch will not work with Clipper.
#include "Mylib.ch" 

#define TAB   CHR(9)
#command QUIT => ( PostQuitMessage( 0 ), __Quit() ) // suggested by Enrico

EXTERNAL DESCEND

REQUEST DBFCDX
REQUEST DBFFPT

STATIC ;
       aBack     := {}         ,;
       cAppTitle := ""         ,;
       cDemoType := "D"        ,;
       cLFile    := "MyApp1"   ,;
       cPath     := ""         ,;
       cPword    := ""         ,;
       cSemaphor := ""         ,;
       hKeyLib   := 0          ,;
       lDemo_Ver := .f.        ,;
       lPx       := .f.        ,;
       nAmt      := 1          ,;
       nCrackChk := 0          ,;
       nLFhandle := 0          ,;
       nSeed     := 0          ,;
       nSemHandle := 0         ,;
       nVersion  := 3          ,;
       nWleft    := 0          ,;
       nWtop     := 0          ,;
       oBrush                  ,;
       oWnd

//---------------------------------------------------------------------------//
  // The app's primary function... (mostly FWH)
FUNCTION Main(cVer)  // cVer identifies which of your apps you are working with
                     // The app can also be ID'd by the name of LF file, as shown below.

   LOCAL hDll

   DEFAULT cVer := "1"

    // This is NOT a PLUS semaphore file - it's an optional extra ini file used
    // on a network to contain a code that momentarily prevents other users 
    // from signing on while someone is "going through the app entry gate."
   cSemaphor := CurDrive() + ":\" +CurDir()+ "\Semaphor.ini" 

*-------------------------
   // CUSTOM CODES: nSeed is set in License File
   nSeed  := 756   // change to whatever # you've decided to use in LF file.
*-------------------------

   aBack  := {79,42,84,35,0,91,56,23,18,67} // returns logic values as numbers
   cPath  := CurDrive()+":\"+CurDir()
   cPword := "putty"    // This password set by you in PLUS V4 license file
   lPx    := .f.
   nWleft := 6
   nWtop  := 3

   RDDSETDEFAULT( "DBFCDX" )

   #ifdef __HARBOUR__
      SET DBFLOCKSCHEME TO 2  //use Comix lock scheme 2; FoxPro is 3
   #endif

   SetHandleCount(101)

   hDLL := LoadLibrary( "BWCC32.dll" ) // Using some Borland controls resources
                                       // - see also the last line of this prg,
   nVersion := VAL(cVer)               // and Freelibrary( hDll) below.

   // The following enables using this prg with several of your apps...
   IF nVersion = 1                   // use nVersion 1 for execution-limit demo
      // The Plus.rc and Plus.dll contain all of the dialog resources used 
      // by this implementation of PLUS. You can copy those resourced into
      // your own app's RC or DLL.
      SET RESOURCES TO "Plus.dll" 
      cLFile := "MyApp1"         
      cAppTitle := "My App 1"     
   ELSEIF nVersion = 2               // use nVersion 2 for date-limit demo
      SET RESOURCES TO "MyApp2.DLL"
      cLFile := "MyApp2"          
      cAppTitle := "My App 2"
   ELSEIF nVersion = 3
      SET RESOURCES TO "MyApp3.DLL"
      cLFile := "MyApp3"
      cAppTitle := "My App 3"
   ELSE
      SET RESOURCES TO "MyApp1.DLL"
      cLFile := "MyApp1"
      cAppTitle := "My App 1"
   ENDIF


   BWCCRegister( GetResources() )

   hKeyLib  := LoadLib32( "KeyLib32.dll" )

   SET _3DLOOK ON

   SetHelpFile("ProtPlus.hlp")

   // Handle years beyond 2000! But allow for birthdates of 80 yr old employees,
   // so that during 2007 all years from '27 through '99 will be treated as in
   // the 1900's and all years from '00 through '26 will be treated as in 2000's.
   // Epoch base date advances automatically each year. 
   IF YEAR(DATE()) < 2099
      SET EPOCH TO (YEAR( DATE() ) - 80)
   ELSE
      SET EPOCH TO 2000 
   ENDIF

   DEFINE BRUSH oBrush COLOR GetSysColor(15)

   DEFINE WINDOW oWnd FROM 3, 6 TO 40, 90 ;
      TITLE "Protection Plus 4.4.1 Test for "+cAppTitle ;
      BRUSH oBrush;
      MENU  BuildMenu()

   @ 2, 2 SAY "Testing Protection Plus 4.4.1" COLORS CLR_BLACK, GetSysColor(15)
   
   ACTIVATE WINDOW oWnd 

   SET RESOURCES TO

   FreeLibrary( hDLL )

   oBrush:End()

   nLFhandle := 0

RETURN nil

//------------------------------- end Main() --------------------------------//
  // Create the app's main menu system... (via FWH)
STATIC FUNCTION BuildMenu()

   local oMenu

   MENU oMenu
      MENUITEM "&Tests Menu"
      MENU
         MENUITEM   "&Run startup test..." ;
            ACTION  Startup() ;
            MESSAGE "Run startup test."

         MENUITEM   "&Do function-limit test..." ;
            ACTION  (aBack := Get_Code(),;
                     nCrackChk := aBack[6],;
                     IF(aBack[8] = 58, lPx := .t., lPx := .f.),;
                     IF(aBack[4] = 63, lDemo_Ver := .t., lDemo_Ver := .f.);
                     ) ;
            MESSAGE "Do function-limit test."

         MENUITEM   "&Network dimensions" ;
            ACTION  (aBack := Get_lanc(), ;
                     nCrackChk := aBack[6],;
                     IF(aBack[8] = 58, lPx := .t., lPx := .f.),;
                     IF(aBack[4] = 63, lDemo_Ver := .t., lDemo_Ver := .f.);
                     ) ;
            MESSAGE "Set network dimensions."

         MENUITEM   "&Reset lic. file demo limit" ;
            ACTION  Reset(nVersion) ;
            MESSAGE "Reset license file demo limit"

         MENUITEM   "&File open test." ;
            ACTION  TestOpen() ;
            MESSAGE "File open test."

         MENUITEM   "&Toggle demo type." ;
            ACTION  ( IF(nVersion = 1,(nVersion := 2,cLFile := "MyApp2"),;
                                      (nVersion := 1,cLFile := "MyApp1") ;
                         ), ;
                      IF(nVersion = 1,MsgInfo("Using executions demo limit."),;
                                      MsgInfo("Using date demo limit.") ;
                         ) ;
                     )     ;
            MESSAGE "Toggle demo type."

         MENUITEM   "E&xit" ;
            ACTION  Exit() ;
            MESSAGE "Exit app."

      ENDMENU
   ENDMENU

RETURN oMenu

//---------------------------------------------------------------------------//
  // Process steps and tests required upon launching the app...
PROCEDURE Startup()
LOCAL ;
      aRay       := {}        ,;
      cDecGama   := SPACE(5)  ,;
      cDecChk    := SPACE(30) ,;
      cDecChk2   := SPACE(30) ,;
      cDecStand  := STR(15)   ,;
      cEncChk    := SPACE(30) ,;
      cEncChk2   := SPACE(30) ,;
      cExpDate   := ""        ,;
      cExpOld    := ""        ,;
      cExpType   := SPACE(10) ,;
      cFile      := ""        ,;
      cInitFile  := ""        ,;
      cStand     := SPACE(15) ,;
      cTGama     := SPACE(5)  ,;
      cTdate2    := CTOD("")  ,;
      cTdatem    := CTOD("")  ,;
      cTenf      := ""        ,;
      cTenFld    := ""        ,;
      dTdate     := CTOD("")  ,;
      dExpold    := CTOD("")  ,;
      nChkDay    := 0         ,;
      nDay       := 0         ,;
      nMo        := 0         ,;
      nRet       := 0         ,;
      nChecksum := 0         ,;
      nTest      := 23        ,;
      nUsers     := 0         ,;
      nYYYY      := 0         

FIELD Gama,Stand,Tenfld

*MsgInfo( "Startup ProPlus tasks." )

// Check to make sure user hasn't tried to replace pppwin.dll with a phony
// dll that returns .t. to all ProPlus functions. Obviously, if pppwin.dll
// has been replaced, then the following ProPlus encryption/decryption
// won't work...


/*

// Test encryption of a session date and the number of executions used
// and storing them in a file...
USE Test1 ALIAS Test1 NEW
GO TOP

pp_decrypt(Stand,"gosh",cStand)

aRay    := Directory("Test1.DBF")
dTdate  := aRay[1,3]         // the file date to be used for the test.
dTdatem := CTOD(RTRIM(cStand))

dTdate2 := Date() + 12       // part of another test if user is date-jockying.
nChkday := DAY(DATE())

cTStand := SPACE(15) 
pp_encrypt(DTOC(date()),"gosh",cTStand)
REPLACE Stand WITH cTStand

cTGama  := SPACE(5)
nTest   := 16
pp_encrypt(STR(nTest+1,2),"gosh",cTGama)
REPLACE Gama WITH cTGama

// Now test decrypting the date and the number from the file...
pp_decrypt(Stand,"gosh",@cDecStand)
pp_decrypt(Gama,"gosh",@cDecGama)

cDecStand := STRTRAN(cDecStand,CHR(0)," ")
cDecGama  := STRTRAN(cDecGama,CHR(0)," ")

cDecStand := RTRIM(cDecStand)

cDecGama  := RTRIM(cDecGama)
*cc
msginfo(cDecStand,"cDecStand")
msginfo(cDecGama,"cDecGama")

CLOSE Test1

*/

IF nVersion = 1
   cFile := cPath+"\MyApp1.lf"     
  ELSEIF nVersion = 2
   cFile := cPath+"\MyApp2.lf"
  ELSEIF nVersion = 3
   cFile := cPath+"\MyApp3.lf"
  ELSE
   cFile := cPath+"\MyApp1.lf"
ENDIF

IF nVersion = 2
   // Open the license file...
 IF pp_lfopen( cFile, LFOPEN_NOCACHE, LF_FILE, cPword , @nLFhandle) = PP_SUCCESS

   pp_getvardate(nLFhandle,VAR_EXP_DATE_SOFT,@nMo,@nDay,@nYYYY)
   cExpDate := STR(nMo,2)+"/"+STR(nDay,2)+"/"+STR(nYYYY,4)
   cExpDate := STRTRAN(cExpDate," ","0")

   MsgInfo(cExpDate,"cExpDate - Lic. File")

   nRet := pp_getvarchar(nLFhandle,VAR_EXPIRE_TYPE,@cExpType)
   msginfo(nRet,"nRet - startup")
   msginfo(cExpType,"cExpType")

   // Close the license file...
   pp_lfclose(nLFhandle)
   nLFhandle := 0

   IF nVersion = 1
       cInitFile := RTRIM(cPath) + "\Wrk\InitFile"
    ELSEIF nVersion = 2
       cInitFile := RTRIM(cPath) + "\Wrk\InitFile"
    ELSEIF nVersion = 3
       cInitFile := RTRIM(cPath) + "\Wrk\InitFil2" // Used when 2 apps share
    ELSE                                           // same data folder, so each can have unique settings.
       cInitFile := RTRIM(cPath) + "\RESET\InitFile"
    ENDIF
  
    IF SELECT("InitFile") = 0
       IF_NETOPEN (cInitFile) ALIAS InitFile SHARED
         ELSE
          Nonopen("InitFile")
          RETURN
       ENDIF
   ENDIF

   cTenfld := Tenfld
   cTenf := SPACE(15)
   pp_decrypt(cTenfld, "wump", @cTenf)
   cTenfld := SUBSTR(cTenf,1,10)
   SET CENTURY ON
   dExpold := CTOD(cTenfld)
*  cExpOld := DTOC(dExpOld)

   MsgInfo(dExpOld,"dExpOld - InitFile")

   CLOSE InitFile

 ENDIF
ENDIF // nVersion = 3

// *************************** \/ \/ \/ HERE IS WHERE PROPLUS.PRG IS TESTED...
aBack := Protplus(cLfile,cPword,nSeed,cDemoType)
nCrackChk := aBack[6]
nSemHandle := aBack[5]
IF(aBack[8] = 58, lPx := .t., lPx := .f.)
IF(aBack[4] = 63, lDemo_Ver := .t., lDemo_Ver := .f.)
IF lPx
   Exit()
ENDIF

*cc
MsgInfo(lDemo_Ver,"lDemo_Ver")

*cc
/*
msginfo(nCrackChk,"nCrackChk")
msginfo(lDemo_ver,"lDemoVer")
msginfo(lPx,"lPx")
*/

RETURN
//-------------------------- end Startup() ----------------------------------//
  // Reset the values in the License File to the initial defaults...
PROCEDURE Reset(nVersion)

      // VERY IMPORTANT: a buffer that will pass back a character
      // value by reference MUST be at least 5 char spaces wider than
      // the field it is getting the data from or else the app will hang up!!
      // So, though cExptype is picking up just 1 char of data, we're
      // giving it a 10 space buffer. (May need to ALLTRIM() the returned data)
LOCAL ;
      cDays      := "  "      ,;
      cDaysMore  := "  "      ,;
      cExpDate   := ""        ,;
      cExptype   := SPACE(10) ,;
      cFile      := ""        ,;
      cInitFile  := ""        ,;
      cStand     := ""        ,;
      cPwstr     := ""        ,;
      cPwstr2    := ""        ,;
      cTGama     := ""        ,;
      cTenf      := ""        ,;
      cTenfld    := ""        ,;
      cTStand    := ""        ,;
      dExpDate   := CTOD("")  ,;
      lAns       := .f.       ,;
      nDay       := 0         ,;
      nDD        := 0         ,;
      nDDsMore   := 0         ,;
      nDaysMore  := 0         ,;
      nExpcount  := 0         ,;
      nMM        := 0         ,;
      nLimit     := 10        ,;  // Put here the desired demo exec. limit
      nRet       := 0         ,;
      nYYYY      := 0         

FIELD Stand,Gama

IF nVersion = 1                 // Doing an executions count test...
   cFile := cPath+"\MyApp1.lf"     
  ELSEIF nVersion = 2           // Same...
   cFile := cPath+"\MyApp2.lf"
  ELSEIF nVersion = 3           // Doing a demo date expiration test...
   cFile := cPath+"\MyApp3.lf"
  ELSE
   cFile := cPath+"\MyApp1.lf"
ENDIF               // Note: can do a function-limit test with any of these.

cPwstr    := "gosh" // Internal password for some aux. encrypted fields
cPwstr2   := "wump" // Internal password for some aux. encrypted fields

   // Open the license file...
nRet := pp_lfopen( cFile, LFOPEN_NOCACHE, LF_FILE, cPword , @nLFhandle)

IF nRet # PP_SUCCESS

    // if there was a problem reading the file, display message and abort

   lAns := MsgAlert("Error # "+ALLTRIM(STR(nRet))+" reading control file A..","Alert")

   RETURN
ENDIF

IF nVersion = 1
   cInitFile := RTRIM(cPath) + "\Wrk\InitFile"
  ELSEIF nVersion = 2
   cInitFile := RTRIM(cPath) + "\Wrk\InitFile"
  ELSEIF nVersion = 3
   cInitFile := RTRIM(cPath) + "\Wrk\InitFil2"
  ELSE
   cInitFile := RTRIM(cPath) + "\RESET\InitFile"
ENDIF

IF_NETOPEN (cInitFile) ALIAS InitFile SHARED
   ELSE
     Nonopen("Wrk\InitFile")
     RETURN
ENDIF

IF InitFile->( Netlock( RECORDLOCK ) )
    ELSE    // Netlock()
     Nonlock("InitFile")
     RETURN 
ENDIF

IF nVersion = 1 .OR. nVersion = 2 
   // Reset executions limit to nLimit and count to 0.
   // So, get Lic. File executions count...
   pp_getvarnum(nLFhandle,VAR_EXP_COUNT,@nExpCount)

   // Reset Lic. File with nLimit executions limit and zero executions count...
   pp_setvarnum(nLFhandle,VAR_EXP_LIMIT,nLimit)
   pp_setvarnum(nLFhandle,VAR_EXP_COUNT,0 )

   // Get updated Lic. File executions count and encrypt it...
   cTGama := SPACE(5)
   pp_getvarnum(nLFhandle,VAR_EXP_COUNT,@nExpCount )
   pp_encrypt(STR(nExpCount,2),cPwstr,@cTGama)

   // Reset InitFile file with encrypted zero executions count...
   Replace Gama WITH cTGama

   // Reset LAN limit to zero.
   pp_setvarnum(nLFhandle,VAR_LAN_LIMIT,0 )

   // Reset demo type to "E"
   pp_setvarchar(nLFhandle,VAR_EXPIRE_TYPE,"E")

   // Remove server from the license file list of unlocked computers...
   pp_copydelete(nLFhandle, COPYDEL_ENHANCED)

   // Note: "Stand" reset near bottom of proc.

ELSEIF nVersion = 2  // Reset demo date...
   // Set expiration type to "demo date"...
   nRet := pp_setvarchar(nLFhandle,VAR_EXPIRE_TYPE,"D")

   // Get Lic. File expiration date...
   pp_getvardate(nLFhandle,VAR_EXP_DATE_SOFT,@nMM,@nDD,@nYYYY)
   cExpDate := STR(nMM,2)+"/"+STR(nDD,2)+"/"+STR(nYYYY,4)
   dExpDate := CTOD(STRTRAN(cExpDate," ","0"))

   MsgGet("Expiration Date","How many days from now to expiration?",@cDaysMore)
   IF EMPTY(cDaysMore)
      MsgAlert("Nothing entered. Will cancel.")
      pp_lfclose(nLFhandle)
      nLFhandle := 0
      RETURN
   ENDIF
   nDaysMore := VAL(cDaysMore)
   SET CENTURY ON
   dExpDate := DATE() + nDaysMore
   cExpDate := DTOC(dExpDate)
   nMM := VAL(SUBSTR(cExpDate,1,2))
   nDay := VAL(SUBSTR(cExpDate,4,2))
   nYYYY := VAL(SUBSTR(cExpDate,7,4))
   pp_setvardate(nLFhandle,VAR_EXP_DATE_SOFT,nMM,nDay,nYYYY)
   // Encrypt new expiration date and save it redundantly for cross-check...
   cTenf := SPACE(15)
   pp_encrypt(cExpDate, cPwstr2, @cTenf)
   cTenfld := SUBSTR(cTenf,1,10)
   SET CENTURY OFF
   REPLACE Tenfld WITH cTenfld

ENDIF

// Reset InitFile file encrypted "last session date" with current date...
cTStand := SPACE(15) 
pp_encrypt(DTOC(date()),cPwstr,@cTStand)
cTStand := SUBSTR(cTStand,1,8)
REPLACE Stand WITH cTStand

InitFile->( dbCommit() )
InitFile->( dbUnLock() )

*cc
IF nVersion = 1 .OR. nVersion = 3
   msginfo(ALLTRIM(STR(nLimit))+" "+ALLTRIM(STR(nExpCount)), "nLimit  nExpCount - Reset()")
ELSE
   msginfo(dExpDate, "dExpDate - Reset()")
ENDIF

nRet := pp_getvarchar(nLFhandle,VAR_EXPIRE_TYPE,@cExptype)

msginfo(cExptype,"cExptype - Reset()")

// Close the license file...
pp_lfclose(nLFhandle)
nLFhandle := 0

RETURN
*------------------------ end Reset() ----
// IMPORTANT: this function MUST be included in your app's main prg,
// so that it can be called from ProPlus.prg in order to get global values...

FUNCTION GetAppStats()
LOCAL aStats := ARRAY(4)

  aStats[1] := oWnd
  aStats[2] := nWtop
  aStats[3] := nWleft
  aStats[4] := cPath

RETURN aStats
*----------------------
  // Mandate app exit...
PROCEDURE Exit()

   IF !EMPTY(oWnd)
      oWnd:End()
      oBrush:End()
   ENDIF

RETURN
//---------------------------------------------------------------------------//
  // The procedure to be performed upon every exit of the app...
EXIT PROCEDURE EndAll()
LOCAL ;
      cFile     := "" 

   cFile := RTRIM(cPath)+"\"+cLFile+".lf"     

*  MsgInfo( "Exit Procedure EndAll()" )

   // Must have next 9 lines anywhere that app is exited, eg. errorsys().
   IF !EMPTY(nSemHandle)
      pp_semclose(nSemHandle)  // close workstation's semaphore.
   ENDIF
   IF pp_lfopen( cFile, LFOPEN_NOCACHE, LF_FILE, cPword , @nLFhandle) = 1
      pp_countdec(nLFhandle,VAR_LAN_COUNT)  // deduct 1 from count of onliners.
      pp_lfclose(nLFhandle)  // close the license file.
   ENDIF
   nLFhandle := 0
   WritePProString("STOPS","Stopq" , "F", cSemaphor ) 

   FreeLib32( hKeyLib )
   ResAllFree()
   SysWait(.1)
   CursorArrow()
   Close Databases
   Clear Memory

RETURN
//---------------------------------------------------------------------------//
  // XBase++ requirement
PROCEDURE AppSys  

RETURN

*-------------------------------
STATIC PROCEDURE TestOpen()

LOCAL cInitFile := ""

cPath  := CurDrive()+":\"+CurDir()

IF SELECT("InitFile") > 0
   CLOSE InitFile
ENDIF

cInitFile := RTRIM(cPath) + "\Wrk\InitFile"

USE (cInitFile) ALIAS InitFile SHARED NEW VIA "DBFCDX" 

/*
IF_NETOPEN (cInitFile) VIA "DBFCDX" ALIAS InitFile NEW SHARED
   ELSE
     Nonopen("Wrk\InitFile")
     RETURN aBack
ENDIF
*/

IF SELECT("InitFile") > 0
   MsgInfo("Initfile was successfully opened.")
   CLOSE InitFile
ELSE
   MsgInfo("Initfile could not be opened.")
ENDIF

RETURN
//---------------------------------------------------------------------------//
  // Required DLL32 function call in order to use Borland resource controls...

DLL32 FUNCTION BWCCRegister( hInst AS LONG  ) AS WORD PASCAL LIB "BWCC32.DLL"

//---------------------------------------------------------------------------//

