*-----------------------------------------------------------------------------
* ProtPlus.PRG                                       R.W. Seiler  Feb 25, 2008
*
*  Implementation of the Protection PLUS library version 4.4.1
*  for use with: FiveWin for Harbour (FWH) and xHarbour
*  
*  For GUI other than FWH, you must change code for the handling of dialog 
*  resources to suit your GUI. Find this code here by searching for the word 
*  RESOURCE. (In 10 code sections.)
*
*  Includes several Optional Code sections (all marked *oc) providing redundant 
*  actions for added complexity. This complexity further frustrates a hacker's 
*  efforts to circumvent this system's security. These optional sections, 
*  using auxiliary data files, can be eliminated if you want a simpler system.
*  Though offering more tactics for frustrating would-be hackers of your app's
*  security, the redundant features have the drawback of also offering more
*  possibilities for app problems.
*
*  Note at bottom of this prg, the layout of the optional auxiliary file.
*-----------------------------------------------------------------------------
/*
 WARNING: Do NOT use FWH's MsgInfo() or MsgAlert() message dialogs in this prg
 because they are nonmodal and can allow the user to bypass security. Only
 use the "RESOURCE MsgOk OF oWnd" dialog, because it is modal and by being tied 
 to oWnd, it helps prevent the user from defeating security restrictions.

 PLUS V4 processes provided in this prg:
      1. Demo implementions via either executions limit, date limit, or app 
         functional limit.
      2. Unlocking of a single user installation, or of a LAN with floating
         workstaion licences. Unlocking interactions between user and provider
         are handled via appropriate screens that are supplied.
      3. In authorizing LAN usage, first the server is unlocked and its ID is
         recorded in the list of licensed machines. Then one workstation is 
         unlocked and the network's ID is added to the list of licensed 
         machines (for use by all workstations, up to the number allowed on at 
         any one time).
      4. Automatic management of demo limits and of licensed rights, including
         control of lan usage.  These controls are handled by routing the user
         through PropLus() on each app launch prior to allowing access to the
         app itself.

 Procedures & Functions in this .PRG :

  // Must call ProPlus() during app's startup. If Proplus() is not called
  // on app startup, then key filewide static values will not be properly set
  // and later calls from app to other funcs here won't work.
 FUNCTION ProtPlus(cLFile,cPword,nSeedIn,nTestIn,nTestOut,cDemoType)
    - Open License File and get file handle for reference.
    - Initial Pro. PLUS processing - lan? how many? single user? demo mode?
    - Read/update InitFile file for auxiliary security functionality.
    - differentiate processing between types of demos: exec limit; demo days;
      or functionality limit.

  // Called from Proplus() if app found to be in demo mode...
 STATIC FUNCTION DemoFunc(nCrackChk,cDate)
    - differentiate processing between types of demos: exec limit or demo days.

  // Single user unlock function called from app directly, from Proplus() or
  // from DemoFunc()...
 FUNCTION Get_Code(nExecleft) 
    - with interactive screen enabling single user unlock on a particular
      computer, or the extension of demo exec limit or days limit.

  // Processing of Get_code(nExecleft) choice...
 STATIC FUNCTION CodeProc( nExecleft, nCode_no, nCompno, nGet_code, nCrackChk, lOkay, oDlgB )

  // Make sure Stand field gets current date and that InitFile filedate
  // will also change to today...
 STATIC PROCEDURE SetStand()

  // Network authorization for # of simultaneous users, called from app...
 FUNCTION Get_LanC()

  // Processing of Get_LanC()choice..
 STATIC FUNCTION LanLimit()

  // Appath encryption and storage...
 STATIC FUNCTION LanLim2(nCode)

  // Check if another lan user can be added, and if so, then do it...
 STATIC PROCEDURE Lanusers()

  // Message screen - LAN users limit has been reached...
 STATIC PROCEDURE LanUsers2(n)

  // Clear counter of lan users count if an app crash left the count high...
 STATIC PROCEDURE ClearUsers()

  // Stopq processing in Semaphor.ini for Lan users...
 STATIC PROCEDURE Leadin(cLFile)

  // Special modal dialog to prevent user from avoiding Pro PLUS constraints...
 STATIC FUNCTION MsgTF(cMsg1,cMsg2,cMsg3)

  // Special modal dialog to prevent user from avoiding Pro PLUS constraints...
 STATIC PROCEDURE MsgOK(cMsg12,cMsg13)

  // Rebuild an index so its date can be used as an auxiliary demo
  // enforcement tool...
 STATIC Procedure BldMyfileIdx()

*/

#include "FiveWin.ch"
#include "KeyLib.ch"
*#include "KeyLibB.ch" // For temp use instead of Keylib.ch to Clipper test-compile in order to see /w warning errors, because keylib.ch will not work with Clipper.
#include "Mylib.ch"  // for custom network functions in Netopen() and Netlock()

       // File-wide variables...
STATIC ;
       aBack     := {}           ,;
       cChkFile  := ""           ,;
       cCdxFile  := ""           ,;
       cCoName   := ""           ,;
       cFax      := ""           ,;
       cFile     := ""           ,;
       cLFile    := ""           ,;
       cDemotype := ""           ,;
       cDrive    := ""           ,;
       cEmail    := ""           ,;
       cStand    := ""           ,;
       cNewGama  := ""           ,;
       cPath     := ""           ,;
       cPhone    := ""           ,;
       cPword    := ""           ,;
       cPwstr    := ""           ,;
       cPwstr2   := ""           ,;
       cPwstr3   := ""           ,;
       cSemaphor := ""           ,;
       cSemPath  := ""           ,;
       cSemPrefx := ""           ,;
       cWebsite  := ""           ,;
       dTdate    := ""           ,;
       dTdate2   := ""           ,;
       dTdatem   := ""           ,;
       lDemo_ver := .f.          ,;
       lDlr      := .f.          ,;
       lGoOn     := .f.          ,;
       lLocalDr  := .t.          ,;
       lN_status := .f.          ,;
       lOnServer := .f.          ,;
       lExit     := .f.          ,;
       lUpdDate  := .f.          ,;
       nChk      := 0            ,;
       nCode_no  := 0            ,;
       nDemFalse := 0            ,;
       nDemoTrue := 0            ,;
       nLFhandle := 0            ,;
       nExFalse  := 0            ,;
       nExTrue   := 0            ,;
       nVersion  := 0            ,;
       nSeed     := 0            ,;
       nSemHandle := 0           ,;
       nTestcnt  := 0            ,;
       nWleft    := 0            ,;
       nWtop     := 0            ,;
       oWnd

*----------------------------
  // Must be called by app during its startup steps...
FUNCTION ProtPlus(cLFil,cPwrd,nSeedIn,cDemoTyp)

FIELD Gama,Stand,Diskpro,Tenfld,Appath,Netname,Orgname,Swt2nam
FIELD Dlrname, Dlrphone, Dlrfax, Dlremail, Dlrwebsite

LOCAL ;
      acNos      := {}        ,;
      aRay       := {}        ,;
      aStats     := {}        ,;
      dExpire    := CTOD("")  ,;
      cAlgorithm := ""        ,;
      cChkFile   := ""        ,;
      cCur_date  := ""        ,;
      cCur_time  := ""        ,;
      cDate      := ""        ,;
      cDateStr   := ""        ,;
      cDay       := ""        ,;
      cDecChk    := SPACE(30) ,;
      cDecrypt1  := SPACE(50) ,;
      cDlrfil    := ""        ,;
      cEncChk    := SPACE(30) ,;
      cExpire    := ""        ,;
      cExpType   := SPACE(10) ,;
      cGet_code  := SPACE(9)  ,;
      cMin       := ""        ,;
      cMsg1      := ""        ,;
      cMsg2      := ""        ,;
      cMsg3      := ""        ,;
      cMsg11     := ""        ,;
      cMsg12     := ""        ,;
      cMsg13     := ""        ,;
      cNewInDat  := ""        ,;
      cPathDecr  := ""        ,;
      cInitFil   := ""        ,;
      cTenf      := SPACE(15) ,;
      cTenfld    := ""        ,;
      cTestWord  := ""        ,;
      cTestPass  := ""        ,;
      cUser      := ""        ,;
      dLastDate  := CTOD("")  ,;
      lAns       := .f.       ,;
      lBuy       := .f.       ,;
      lFwrd      := .f.       ,;
      lGoAhead   := .f.       ,;
      lOk        := .f.       ,;
      nCe_num    := 0         ,;
      nCnt       := 0         ,;
      nCompno    := 0         ,;
      nDD        := 0         ,;
      nDiskpro   := 0         ,;
      nGet_code  := 0         ,;
      nHr        := 0         ,;
      nMin       := 0         ,;
      nMM        := 0         ,;
      nNetname   := 0         ,;
      nNetserve  := 0         ,;
      nNetEnhan  := 0         ,;
      nRet       := 0         ,;
      nSec       := 0         ,;
      nSfcnt     := 0         ,;
      nTestBack  := 0         ,;
      nTestIn    := 0         ,;
      nTestOut   := 0         ,;
      nUsers     := 0         ,;
      nYYYY      := 0         ,;
      oDlgOK                  ,;
      oMsg1                   ,;
      oMsg2                   ,;
      oMsg3                   ,;
      oMsg11                  ,;
      oMsg12                  ,;
      oMsg13                  ,;
      oPhone                  ,;
      Settng     := "" 

STATIC nCrackChk := 0

DEFAULT nSeedIn  := 0  
DEFAULT cPwrd    := "" 
DEFAULT cLFil    := ""
DEFAULT cDemoTyp := "E"  // PLUS4 expiration types are N,E,S,P and D. Each of
                         // these provides a type of demo limitation, e.g.
                         // "E" is a limit on allowed program Executions, and
                         // "D" imposes a Date limit, like 30 days. Each type
                         // of demo limitation is handled differently. This prg
                         // uses an added type "F" to indicate a "Functional"
                         // limition in the use of a demo version. Example:
                         // a training course with 20 lessons could give
                         // access to the first lesson as a free demo, but
                         // require purchase of an unlock code to gain access
                         // to the remaining 19 lessons. In this case, the
                         // training app would sense when the user had
                         // completed the first lesson, and wouldn't allow
                         // further progress until the user clicked an "Unlock"
                         // button to facilitate getting an unlock code, and
                         // then had entered the correct unlock code.
                         // "N" = None (unlocked); "S" = Shareware;
                         // "P" = Payment limit.

****************************************************************
*------------------- BEGIN ProPlus CUSTOM DATA SET -------------

CursorWait()

// These filewide statics and a few locals are set when Proplus() is called 
// during app startup. All of these are customizable values except 
// nTestIn/nTestOut, which are assigned to you by Softwarekey.com...

cLFile    := "MyApp"    // Default license file name w/o name extension

      // The values for these 2 codes have been uniquely assigned to you by 
      // Softwarekey.com and MUST be set here and used for all your apps. 
      // These codes are used in verifying Keylib32.dll file validity at 
      // runtime (so that someone else can't use their Keylib32.dll with
      // your apps)...
nTestIn   := 1234567890 // Imbedded in PLUS V4; obtained from Softwarekey.com
nTestOut  := 1234567    // Imbedded in PLUS V4; obtained from Softwarekey.com
nSeed     := 756        // The "seed" set by you in PLUS V4 license file
cPword    := "putty"    // This password set by you in PLUS V4 license file
      // The above 2 codes are set in each app's license file and can be
      // either the same or different for each app.  If different for each
      // app, then set the default values here for your first app. Then 
      // for other apps, pass the values as ProPlus() parameters.  This way
      // you only need to maintain one version of ProPlus.prg for all apps.

aBack     := {79,42,84,35,0,91,56,23,18,67} // Returns logic values as numbers
                          // in order to defeat would-be cracks. Also, aBack[5]
                          // carries nSemhandle for the user's semaphore file,
                          // needed to close the file on user's exit.

nCrackChk := 83           // Optional value used to defeat would-be cracks.
aBack[6]  := nCrackChk        // Optional value used to defeat would-be cracks.

nDemFalse := 41    // Used for aBack[4] to indicate for lDemo_ver
nDemoTrue := 63    // Used for aBack[4] to indicate for lDemo_ver
nExTrue   := 58    // For aBack[8] to indicate for optional lExit (to abort app)
nExFalse  := 26    // For aBack[8] to indicate for optional lExit (app continues)

cNewInDat := "04/15/2001" //Generic new install date indicator in Tenfld field.
                   // This date found in aux data file means this is a new
                   // installation, and it gets set to current date on first
                   // launch of app.

cTestWord := "halleluja" // Password to check validity of Keylib32.dll
cTestPass := "24162" // Internal passwrd for some aux. encrypted flds - 5 char!
cPwstr    := "gosh"  // Internal password for some aux. encrypted fields - 4 char
cPwstr2   := "wump"  // Internal password for some aux. encrypted fields - 4 char
cPwstr3   := "8502"  // 4 char. numbers

cCoName   := "XYZ Software Corp."  // Your company name goes here
cPhone    := "212-123-4567"             // your co. phone #

IF cLFil = "MyApp1"                // Your license filename prefix - 1st prod
   nVersion := 1
ELSEIF cLFil = "MyApp2"            // License filename prefix - your 2nd prod
   nVersion := 2
ELSE
   nVersion := 3
ENDIF

*------------------- END OF ProPlus CUSTOM DATA SET -----------
***************************************************************

CLOSE DATABASES

// Load key filewide statics with parameter values if any values are passed in
// to override default custom values hard-coded in section just above...
IF !EMPTY(cLFil)
   cLFile    := cLFil
ENDIF
IF !EMPTY(cPwrd)
   cPword    := cPwrd
ENDIF
IF !EMPTY(nSeedIn)
   nSeed := nSeedIn
ENDIF

cDemotype := cDemotyp     // Demotyp param defaults to "E" if nothing passed in
cFile     := cLFile+".lf" // Default license file name with name extension

acNos     := {"0","1","2","3","4","5","6","7","8","9"}

lOnServer := .f.
lUpdDate  := .f.


// This Semaphor.ini is not a PLUS file - it is part of the redundant features
// included in this prg for added utility and security...
cSemaphor := CurDrive() + ":\" +CurDir()+ "\Semaphor.ini" 
 // Note: must use entire path in the defining cSemaphor, or else
 // Keylib32.dll will look for it in the Windows directory.

 // Get global values from app's main prg...
 // Note: MUST add the GetAppStats() function to the app's main prg. Copy this
 //       function from PlusTst1.prg.
aStats    := GetAppStats()
oWnd      := aStats[1] // App's main window object.
nWtop     := aStats[2] // The main window's top position.
nWleft    := aStats[3] // The main window's left position.
cPath     := aStats[4] // The app's path location on the hard drive.

cDrive := SUBSTR(cPath,1,1)

IF nVersion = 3
   cChkfile := rtrim(cPath) + "\Wrk\InitFil2.dbf"
ELSE
   cChkfile := rtrim(cPath) + "\Wrk\InitFile.dbf"
ENDIF

cCdxFile := "Yourfile.cdx"


dTdate    := CTOD("")  // Last mod date of JOURNAL file from disk dir.
dTdate2   := CTOD("")  // DATE() + 12 (used to check for user's date jockeying)
dTdatem   := CTOD("")  // Date converted from InitFile Stand (char) field.

lN_status := .f.       // If app is running on a network.
lLocalDr  := .t.       // If app is running on a local hard drive.
lDlr      := .f.       // If this copy of app is serviced through a dealer.
nChk      := 0         // This will get # of program executions made.

//               *****    PRO-PLUS APP STARTUP PROCESSING    *****
CursorWait()
// Must check to make sure user hasn't tried to replace Keylib32.dll with a 
// phony dll that returns .t. to all PLUS V4 functions. Obviously, if 
// Keylib32.dll has been replaced, then the PLUS V4 encryption/decryption
// used in the 2nd test here won't work...

// First, use a built-in PLUS V4 way to check this...

nTestBack := pp_libtest( nTestIn )

IF nTestBack # nTestOut
   MsgOK("Corrupted DLL file.","Phone technical support",cPhone)
   lExit := .t.
   aBack[8] := nExTrue
   RETURN aBack
ENDIF

// Now, a second way to check for phony dll, using the Keylib32.dll encryption
// functions. If the user has substituted this dll with a phony, then this 
// encryption test will fail...

pp_encrypt( cTestWord, cTestPass, @cEncChk )

pp_decrypt( cEncChk, cTestPass, @cDecChk )

IF ALLTRIM(cDecChk) # cTestWord
   MsgOK("Corrupted master file.","Phone technical support",cPhone)
   lExit := .t.
   aBack[8] := nExTrue
   RETURN aBack
ENDIF


cFile := RTRIM(cPath)+"\"+cLFile+".lf"     

IF !FILE(cFile)
   MsgOK("License file is missing.","Phone tech. support: "+cPhone)
   lPx := .t.
   aBack[8] := nExTrue
   RETURN aBack
ENDIF


// NOTE: Interspersed below are some code sections with "optional features"
// that utilize some extra files to reduntantly store information that
// is stored in the PLUS license file.  The redundancy is used to crosscheck
// the license file data to make it less likely that a hacker will be able to
// defeat the demo limits by setting his system date back and then replacing
// the license file with an earlier version that would allow more demo time.
// Doing so would contradict the redundant data stored in the extra files.
// This cross-checking also strengthens copy protection by complicating the
// hacker's job, even if the hacker is familiar with the PLUS system.

*op          // optional features marked with *op for easy search
// OPTIONAL FEATURES, set A...
cCoName  := "XYZ Software Corp."  // Your co. name
cPhone   := "123-456-7890"             // Your co. phone
cFax     := "."
cEmail   := "."
cWebsite := "."

IF FILE( RTRIM(cPath) + "\RESET\Dealer.dbf" )
  cDlrfil := RTRIM(cPath) + "\RESET\Dealer"
  // Using network functions Netopen() & Netlock() as provided by Larry Dysert 
  // in Clipper Advisor, in conjunction with MyLib.ch.
  IF_NETOPEN (cDlrfil) ALIAS Dealer SHARED
    cCoName  := Dlrname
    cPhone   := Dlrphone
    cFax     := Dlrfax
    cEmail   := Dlremail
    cWebsite := Dlrwebsite
    lDlr     := .t.
    CLOSE Dealer
  ELSE
    NONOPEN("DEALER")
    cCoName  := "XYZ Software Corp."  // Your company info...
    cPhone   := "123-456-7890"
    cFax     := "123-456-7777"
    cEmail   := "info@XYZsoft.com"
    cWebsite := "http://www.XYZsoft.com"
    lDlr     := .f.
  ENDIF
 ELSE
  cCoName  := "XYZ Software Corp."   // Your company info...
  cPhone   := "123-456-7890"
  cFax     := "123-456-7777"
  cEmail   := "info@XYZsoft.com"
  cWebsite := "http://www.XYZsoft.com"
  lDlr     := .f.
ENDIF

IF SELECT("InitFile") > 0
   CLOSE InitFile
ENDIF

IF nVersion = 1
   cInitFil := RTRIM(cPath) + "\Wrk\InitFile"
  ELSEIF nVersion = 2
   cInitFil := RTRIM(cPath) + "\Wrk\InitFile"
  ELSEIF nVersion = 3
   cInitFil := RTRIM(cPath) + "\Wrk\InitFil2"
  ELSE
   cInitFil := RTRIM(cPath) + "\RESET\InitFile"
ENDIF

   // This is the auxiliary file most used for optional features...
IF_NETOPEN (cInitFil) VIA "DBFCDX" ALIAS InitFile NEW SHARED
   ELSE
     Nonopen("Wrk\InitFile")
     pp_lfclose(nLFhandle)
     nLFhandle := 0
     RETURN aBack
ENDIF

// END OF OPTIONAL section A.

lExit := .f.  // This app global value tells the calling Startup() whether
            // to exit immediately or allow access to the full app.
            // If lExit = .f. , okay to proceed; if .t. then must exit.
aBack[8] := nExFalse

  // If someone else is starting their entry into the app right now,
  // delay entry until other user is through the entry gate:
Leadin(cLFile)

IF lExit = .t.    // If Leadin() action is "cancel."
   aBack[8] := nExTrue
   WritePProString("STOPS","Stopq" , "F", cSemaphor ) 
   RETURN aBack
ENDIF

cStand   := SPACE(15)
cNewGama := SPACE(5)

SysRefresh()
CursorWait()

****************************** OPEN LF...

   // OPEN THE LICENSE FILE...

nRet := pp_lfopen( cFile, LFOPEN_NOCACHE, LF_FILE, cPword , @nLFhandle)

IF nRet # PP_SUCCESS

    // If there was a problem reading the file, display message and abort...

    cMsg1 := "Error # "+ALLTRIM(STR(nRet))+" reading control file A."
    cMsg2 := "Call tech support " + cPhone
    cMsg3 := "Are they on phone with you now?"

    lAns := MsgTF(cMsg1,cMsg2,cMsg3)

    IF lAns = .t.
       lFwrd := .f.
       lN_status := .f.
       lLocalDr  := .t.
       if pp_redir( cDrive ) = 1 // = .t. 
                                           
         lN_status = .t.                  
         lLocalDr := .f.                  
       endif                              
       aBack := Get_Code(0)
       nCrackChk := aBack[6]
    ENDIF
    lDemo_ver := .t.
    lExit := .t.
    aBack[4] := nDemoTrue
    aBack[8] := nExTrue
    CLOSE DATABASES
    pp_lfclose(nLFhandle)
    nLFhandle := 0
    WritePProString("STOPS","Stopq" , "F", cSemaphor ) 
    RETURN aBack
ENDIF  // nRet

SELECT InitFile

CursorWait()

  // Check if last app usage date already has today's (or later) date...
pp_getvardate(nLFhandle,VAR_LAST_DATE,@nMM,@nDD,@nYYYY)
dLastDate := CTOD(STR(nMM,2)+"/"+STR(nDD,2)+"/"+SUBSTR(STR(nYYYY,4),3,2))
IF dLastDate >= DATE()
   lUpdDate := .t.    // ...to prevent time wasting call to pp_upddate() later.
ENDIF

*op
// OPTIONAL FEATURES SECTION B...

// The optional features reduntantly store demo and unlock data in auxiliary
// dbf files in order to further complicate the difficulties faced by any
// hacker who tries to defeat the Plus licensing system.

CursorWait()

// END OPTIONAL section B.

   // Get demo expiration type...
nRet := pp_getvarchar(nLFhandle,VAR_EXPIRE_TYPE,@cExpType)
cExpType := SUBSTR(cExpType,1,1)

// Initialize network status to true as the default...

lN_status := .f.
lLocalDr  := .t.
// test for drive redirection (network drive); "0" = default drive
IF pp_redir( cDrive ) = 1 // = .f.   // DOS ver: This done in QNETCHK.EXE, passing result to QNET.MEM
   lN_status = .t.
   lLocalDr := .f.
ENDIF

*op
  // Redundant net check, as PPP V3 pp_redir() apparently doesn't work with 
  // XP Pro. Unknown if V4 pp_redir() always works with XP Pro, so still using
  // this workaround as a backup... (Maybe problem was use of pp_redir("0") instead of pp_redir(cDrive) ).
*IF CurDrive() > "D"  
*   lN_status := .t. 
*   lLocalDr  := .f.
*ENDIF

CursorWait()

IF lLocalDr        // We're not on a network workstation, just on a 
                   // single computer or peer server...

  CursorWait()

                   // This must be 0 or else legacy function used.
     // Determine if in DEMO Mode...            \/
  IF pp_copycheckth(nLFhandle, COPYCHK_ENHANCED, 0, 20) = 1
     lDemo_ver := .f. // This computer already unlocked.
     aBack[4]  := nDemFalse
  ELSE
     lDemo_ver := .t. // Use demo mode for this computer.
     aBack[4]  := nDemoTrue
  ENDIF

  IF lDemo_ver = .t. 

     // Check for demo expiration by executions limit...
     IF cExpType = "E"

      nRet := pp_setvarnum(nLFhandle, VAR_LAN_LIMIT, 0 )
                   // So only server can use program in demo mode.
                   // Otherwise, a client would have unlim access in demo mode.
                   // Server's nCompno (enhanced) can't be seen by client,
                   // so demo can't be used at client station.  
      SysRefresh()
      CursorWait()

      aBack := DemoFunc(nCrackChk,cDate) 
      nCrackChk := aBack[6]

     // If applicable, check for demo expiration by date limit...
     ELSEIF cExpType = "D"

*op
      // OPTIONAL REDUNDANT FEATURES section C, to strengthen security...
      nMM   := 0
      nDD   := 0
      nYYYY := 0
      SELECT InitFile   
                        
         // A redundant system to make it harder to cheat...
      cTenfld := Tenfld
      cTenf := SPACE(15)
      pp_decrypt(cTenfld, cPwstr2, @cTenf)
      cDate := SUBSTR(cTenf,1,10)
      cDate := STRTRAN(cDate," ","0")
      SET CENTURY ON

        // If this is a fresh new installation indicated by cNewInDat date 
        // found in the InitFile file Tenfld field, then record the appropriate
        // initial demo expiration date...
      IF cDate == cNewInDat
         dExpire := DATE() + 30
         cExpire := DTOC(dExpire)
         nMM     := MONTH(dExpire)
         nDD     := DAY(dExpire)
         nYYYY   := YEAR(dExpire)
         cDate   := RTRIM(cExpire)
         cTenf   := SPACE(15)
         pp_encrypt(cExpire, cPwstr2, @cTenf)
         cTenfld := SUBSTR(cTenf,1,10)
         cStand  := SPACE(15) 
         SET CENTURY OFF
         pp_encrypt(DTOC(date()),cPwstr,@cStand)
         cStand  := SUBSTR(cStand,1,8)
         IF InitFile->( NetLock( RECORDLOCK ) ) 
            // Tenfld contains demo expiration date - redundant for cross-check
            REPLACE Tenfld WITH cTenfld
            // Stand contains date of most recent session for future check
            // against date jockeying to defeat demo expiration...
            REPLACE Stand WITH cStand   
           ELSE
              Nonlock("InitFile")
              lExit := .t.
              aBack[8] := nExTrue
              CLOSE DATABASES
              WritePProString("STOPS","Stopq" , "F", cSemaphor ) 
              RETURN aBack
         ENDIF
         InitFile->( dbCommit() )
         InitFile->( dbUnLock() )

         // REDO MEMBERS.CDX HERE!!!
         // Do only for TEMAINT.EXE - date-based demo.
         BldMyfileIdx()  // Provides .CDX file date check to prevent "crack"
            // If date of MEMBERS.CDX is more recent than date of InitFile.DBF
            // then assume user has attempted to
            // use old InitFile.DBF to extend demo, so generate error.
            // Similar check done with TENGINE.EXE date.
            // MUST do BldMyfileIdx() whenever \RESET\InitFile.DBF updated so
            // that legal filedates will match.

            // Must reopen InitFile - closed by BldMyfileIdx()
         IF SELECT("InitFile") = 0
            IF_NETOPEN (cInitFil) ALIAS InitFile SHARED
              ELSE
               NONOPEN("\RESET\InitFile")
               lExit := .t.
               aBack[8] := nExTrue
               CLOSE DATABASES
               WritePProString("STOPS","Stopq" , "F", cSemaphor ) 
               RETURN aBack
            ENDIF
         ENDIF
         
      ENDIF  // If cDate = cNewInDat, indicating a new installation.
      // END OF OPTIONAL section C.

            // Is this the right place for this date-limit version call?????...
      aBack := DemoFunc(nCrackChk,cDate)

      nCrackChk := aBack[6]

*op
      // OPTIONAL REDUNDANT FEATURES section D, to strengthen security...
      IF SELECT("InitFile") = 0
         IF_NETOPEN (cInitFil) ALIAS InitFile SHARED
            ELSE
               NONOPEN("\RESET\InitFile")
               lExit := .t.
               aBack[8] := nExTrue
               CLOSE DATABASES
               WritePProString("STOPS","Stopq" , "F", cSemaphor ) 
               RETURN aBack
         ENDIF
      ENDIF
      SELECT InitFile   // Workaround, as couldn't get prior Plus V3 wrapper
                        // to update expiration date field...
      cTenfld := Tenfld 
      cTenf := SPACE(15)
      pp_decrypt(cTenfld, cPwstr2, @cTenf)
      cDate := SUBSTR(cTenf,1,10)
      cDate := STRTRAN(cDate," ","0")

      IF CTOD(cDate) >= Date() .OR. lGoOn
         cDateStr := SUBSTR(CMONTH(CTOD(cDate)),1,3 ) + ;
                         STR(DAY(CTOD(cDate))) + ", " + ;
                         STR(YEAR(CTOD(cDate))) 
         lFwrd := .t. // So Stand date will be updated in InitFile to today

         // Notice to user of expiration date...
         MsgOk(,"Demo expiration date: " + cDateStr)

        ELSE

         lBuy := MsgTF("Purchase unlock code for regular use?")

         IF lBuy
            IF lDemo_ver
               Get_code(0)
              ELSE
               MsgAlert("Program is already unlocked.")
            ENDIF
         ENDIF
      ENDIF
      // END OF OPTIONAL section D.

         // If demo period has expired...
         // (Workaround - still used - due to PPPWIN V3 wrapper date problem)
      IF DATE() > CTOD(cDate) .AND. lDemo_ver = .t. 
         pp_upddate(nLFhandle, UPDDATE_NOTIME )  
              // Update control file with system date to indicate
              // last time app used. This prevents defeating expiration
              // date by setting system date back after seeing expir. notice.
         lUpdDate := .t.

         SET CENTURY OFF

*op
         // OPTIONAL SECTION E using aux. file data to enhance security...
         cStand  := SPACE(15)
         pp_decrypt(Stand,cPwstr,@cStand)

          // Optional check-date...
*         aRay    := Directory(rtrim(cPath) + "\" + cCdxFile )
*         dTdate  := aRay[1,3]

         dTdatem := CTOD(SUBSTR(cStand,1,8))
         dTdate2 := Date() + 12

            // Protects Stand from change if 
            // innocent short-term date jockeying...
         IF dTdatem < Date() .OR. dTdatem > dTdate2 
            cStand := SPACE(15) 
            pp_encrypt(DTOC(date()),cPwstr,@cStand)
            cStand := SUBSTR(cStand,1,8)
            IF InitFile->( NetLock( RECORDLOCK ) ) 
                // This duplicates _pp_upddat() functionality...
                REPLACE Stand WITH cStand
              ELSE
                Nonlock("InitFile")
                lExit := .t.
                aBack[8] := nExTrue
                CLOSE DATABASES
                WritePProString("STOPS","Stopq" , "F", cSemaphor ) 
                RETURN aBack
            ENDIF
            InitFile->( dbCommit() )
            InitFile->( dbUnLock() )
         ENDIF
         // END OF OPTIONAL section E.

         MsgOk("Sorry, but the demo for this product has expired.",;
               "Please call and order from:", cCoName + " at " + cPhone, .t. ) // .t. = "Alert:"
         lExit := .t.
         aBack[8] := nExTrue
         WritePProString("STOPS","Stopq" , "F", cSemaphor ) 
         RETURN aBack
      ENDIF

         // Make sure last date and time used is less than current date/time.
         // If not, force reset of system date/time...
      IF cExpType # "N" .AND. pp_valdate(nLFhandle) = 0  // "N"= exp typ "none"
         cCur_date := DTOC(date())
         cCur_time := time()
         lOk := MsgGet("Reset System Date","Date mm/dd/yy: ",@cCur_date)
         lOk := MsgGet("Reset System Time","Time hr:min:sec - ",@cCur_time)
         IF lOK
            nMM   := VAL(SUBSTR(cCur_date,1,AT("/",cCur_date)-1))
            cDay  := SUBSTR(cCur_date,AT("/",cCur_date)+1,2)
            nDD   := VAL(STRTRAN(cDay,"/",""))
            nYYYY := VAL(SUBSTR(cCur_date,RAT("/",cCur_date)+1,2))

            SetDate( nMM, nDD, nYYYY )

            nHr  := VAL(SUBSTR(cCur_time,1,AT(":",cCur_time)-1))
            cMin := SUBSTR(cCur_time,AT(":",cCur_time)+1,2)
            nMin := VAL(STRTRAN(cMin,":",""))
            nSec := VAL(SUBSTR(cCur_time,RAT(":",cCur_time)+1,2))

            SetTime( nHr, nMin, nSec )

         ENDIF
      ENDIF

         // Second check for valid date/time...
      IF cExpType # "N" .AND. pp_valdate(nLFhandle) = 0 // "N" = exp type "none"
                                                        //       (unlocked)
         nCe_num := pp_cenum()

         MsgOk("System date and time still incorrect.", ;
               "Please call Tech. Support for assistance.", ;
               "Code entry #: " + ltrim( str( nCe_num ) ) + " Date: " + DTOC( date() ),.t. ) // .t. = "Alert:"

         cGet_code := SPACE(9)
         lOk := MsgGet("Code Entry","Enter numeric code: ", @cGet_code)
         nGet_code := VAL(cGet_code)
         nCompno   := pp_compno( COMPNO_ENHANCED, "", "" )

         IF pp_tcode( nGet_code, nCe_num, nCompno, nSeed ) = 5
            pp_upddate(nLFhandle, UPDDATE_NOTIME)
            lUpdDate := .t.
         ELSE
            lExit := .t.
            aBack[8] := nExTrue
            WritePProString("STOPS","Stopq" , "F", cSemaphor ) 
            RETURN aBack
         ENDIF

      ENDIF

     ENDIF

    ELSE   // On local drive, but not demo...

      CursorWait()

        // Though drive not redirected, is LAN usage active for this app?...
      IF pp_lanactive(nLFhandle) = 1

        // We only get here if app is being run directly on the server...

        lOnServer := .t.

        aRay := {}
        IF FILE(cInitFil+".dbf")
           aRay := Directory(cInitFil+".dbf")
        ELSE
           MsgAlert( cInitFil+".dbf"+" file not found. Will cancel." )
           lDemo_ver := .t.
           aBack[4]  := nDemoTrue
           nCrackChk := 4
           aBack[6]  := nCrackChk
           lExit     := .t.
           aBack[8]  := nExTrue
           RETURN aBack
        ENDIF

        dTdate := aRay[1,3]

        IF dTdate < DATE() .OR. !lUpdDate
                           // If server makes first startup of the day it will
                           // clear the current users count in case prior crash
                           // caused count not to decrement on exit for a user.

           IF SELECT("User") = 0
             // USER.DBF is a dummy file used to make sure that reset of users count
             // can only be done when everyone else is signed off.

             IF nVersion = 3
                cUser := "User360"
             ELSE
                cUser := "User"
             ENDIF
             IF_NETOPEN (cUser) ALIAS User EXCLUSIVE
                pp_setvarnum(nLFhandle, VAR_LAN_COUNT, 0 )
             ENDIF
           ENDIF

           SysRefresh()
           CursorWait()

        ENDIF

        // Static proc to check if another user can enter...
        LanUsers(1)

        CursorWait()

        IF lExit = .t.               // Superfluous??
           aBack[8]  := nExTrue
           lDemo_ver := .t.
           aBack[4]  := nDemoTrue
           pp_lfclose(nLFhandle)
           nLFhandle := 0
           WritePProString("STOPS","Stopq" , "F", cSemaphor ) 
           RETURN aBack
        ENDIF
      ENDIF  // pp_lanactive()
  ENDIF     // if lDemo_ver

 ELSE    // !lLocalDr  -   NETWORK  -   NETWORK - on a redirected drive...

    CursorWait()

    lOnServer := .f.  // = the same meaning as !lLocalDr, but lOnServer=.t. is 
                      // not necessarily the same as lLocalDr=.t., which can 
                      // apply to both a server OR a single user installation.
                      // lOnServer=.t. on the other hand applies ONLY to use
                      // on a server.

    // We are running on a network drive.
/*
    // If app is "RSO" on a network, then automatically set to DEMO mode:
    //          ("RSO" = Run Single-computer Only)

    lDemo_ver := .t.
    aBack[4]  := nDemoTrue

    MsgOk("This version of the program will run",;
          "only in demo mode on a network. Contact",;
          "XYZ Software Corp. to upgrade.", .t. ) // .t. = "Alert:"

*/
    // Now see if this application is ALLOWED to run on a network, 2 ways...

    // On peer clients, set copy protection ID algorithm to COMPNO_NETNAME (16)
    // Note: COMPNO_NETNAME only works right if the appropriate drive letter
    //       is passed as a param with it.
   cAlgorithm = COMPNO_NETNAME

    // Identify this computer...
   nCompno = pp_compno(cAlgorithm, "", cDrive )

   CursorWait()

    // See if we are authorized to run on a network workstation...
   if pp_copycheck(nLFhandle, 0, nCompno) # 1  // any failure of some sort

       lGoAhead := .f.  // How are we using this?
       MsgAlert("This application must be net-enabled to allow "+CRLF+;
                "use on a network workstation. Please contact "+CRLF+;
                cCoName + " at: "+cPhone, "Network Activation")
       MsgInfo("First step: unlock this workstation, followed by,"+CRLF+CRLF+;
               "Second step: authorize network dimensions.","Steps to Enable Network" )
       lDemo_ver := .t.
       aBack := Get_Code(0)  // aBack is FW static, so should it be returned?
       IF !lDemo_ver // Workstation was successfully unlocked, so do LAN wrk...
          aBack := Get_Lanc()
       ENDIF
       // Force exit from app so they'll do a clean re-launch...
       lExit := .t.
       aBack[8] := nExTrue
       WritePProString("STOPS","Stopq" , "F", cSemaphor ) 
       RETURN aBack
   endif

   // See if we are also authorized for a given number of floating workstns...
   IF pp_lanactive(nLFhandle) = 1  // = True  "Is lan limit > 0 ?" 

*op
          // OPTIONAL REDUNDANCY - can only be run from a workstation...
          // Make sure user hasn't copied an unlocked net version into
          // multiple directories on the same machine in order to get more
          // user rights illegally...
       IF !lOnServer
          cDecrypt1  := SPACE(50)
          pp_decrypt( Appath, cPwstr3, @cDecrypt1 )
          cPathDecr := SUBSTR(cDecrypt1,1,40)

/*
          // "initnewnet" is stored in Appath when field created by Filesmod.
          // InitFile file newly modified with this feature
          // in Filesmod.prg - Setcopy()            
          //                                     \/ Last term needed for net custs where cPath was used here.
          IF ALLTRIM(cPathDecr) == "initnewnet" .OR. ASCAN(acNos, SUBSTR(cPathDecr,1,1)) = 0
           // Insert server's "Netname" stringified code number into Appath...

           // This step must eventually be eliminated after all net users as of
           // 12/00 have been upgraded. Otherwise, someone could eventually
           // figure how to defeat system by copying an old InitFile.dbf file
           // into Wrk dir. This step just to accomodate existing net users.

             cPathDecr := LanLim2(7)

          ENDIF
*/
          // Only do this from a workstation. If done from server, won't get
          // correct nNetname.
          nNetname := pp_compno( COMPNO_NETNAME , "" , cDrive  ) // legacy version of pp_compno()

          // This had caused a problem with legal setups - probably was due
          // to running on the server instead of a worksation...
          IF nNetname # VAL(cPathDecr)
             MsgOK("Invalid network setup.", "Phone technical support:",;
                   "1-212-123-4567")
             lExit := .t.
             aBack[8] := nExTrue
             WritePProString("STOPS","Stopq" , "F", cSemaphor )     
             CLOSE DATABASES
             pp_lfclose(nLFhandle)
             nLFhandle := 0
             RETURN aBack
          ENDIF

       ENDIF // !lOnServer

       aRay := {}

       IF FILE(cChkfile)
          aRay := Directory(cChkfile)
       ELSE
          MsgAlert( cChkfile + " file not found. Will cancel." )
          lDemo_ver := .t.
          aBack[4]  := nDemoTrue
          nCrackChk := 4
          aBack[6]  := nCrackChk
          lExit     := .t.
          aBack[8]  := nExTrue
          RETURN aBack
       ENDIF

       dTdate := aRay[1,3]

       IF dTdate < DATE() .OR. !lUpdDate 
                    // If this is first startup of the day, then
                    // clear the current users count in case prior crash
                    // caused count not to decrement on exit for a user.
          IF SELECT("User") = 0
             // USER.DBF is a dummy file used to make sure that reset of users count
             // can only be done when everyone else is signed off.

             IF nVersion = 3
                cUser := "User360"
             ELSE
                cUser := "User"
             ENDIF
             IF_NETOPEN (cUser) ALIAS User EXCLUSIVE
                pp_setvarnum(nLFhandle, VAR_LAN_COUNT, 0 )
             ENDIF
          ENDIF

       ENDIF

       SysRefresh()
       CursorWait()

       LanUsers(2)

       IF lExit = .t.               // Superfluous??
          aBack[8] := nExTrue
          WritePProString("STOPS","Stopq" , "F", cSemaphor ) 
          pp_lfclose(nLFhandle)
          nLFhandle := 0
          RETURN aBack
       ENDIF

       // If we got here, the program is authorized to run.  Proceed to the
       // power-up screen.

    ELSE     // pp_lanactive() = 0 even though on a redirected drive...

        // Note to RWS: old ProplusD.prg had a call to DemoFunc() here if
        // app's .drv file didn't exist.

        // Not yet allowed to run on a network drive...
        cMsg11 := "Network use not yet licensed."
        cMsg12 := "Please phone customer service at"
        cMsg13 := "1-212-123-4567 for information."

        DEFINE DIALOG oDlgOK RESOURCE "MsgOK" of oWnd

        REDEFINE SAY oMsg11 PROMPT cMsg11 ID 21 OF oDlgOK
        REDEFINE SAY oMsg12 PROMPT cMsg12 ID 22 OF oDlgOK
        REDEFINE SAY oMsg13 PROMPT cMsg13 ID 23 OF oDlgOK

        REDEFINE BUTTON ID 11 OF oDlgOK ;   // Yes
          ACTION ( oDlgOK:End(), Get_LanC() )

        ACTIVATE DIALOG oDlgOK CENTERED ;
          ON INIT (SetFocus(oDlgOK)) RESIZE16

        CLOSE DATABASES
        WritePProString("STOPS","Stopq" , "F", cSemaphor )     
        lExit := .t.
        aBack[8] := nExTrue
        pp_lfclose(nLFhandle)
        nLFhandle := 0
        RETURN aBack

   ENDIF    // pp_lanactive()

ENDIF       // if lLocalDr

// Make sure Stand field gets current date and that InitFile filedate
// will also change to today. This is needed in order to automate 
// ClearUsers()...

SetStand()

// Open the gate for next user to enter...
WritePProString("STOPS","Stopq" , "F", cSemaphor )     

pp_lfclose(nLFhandle)  // close the license file.
nLFhandle := 0

SysWait(.2)
CursorArrow()

// The return value tells the app if it is allowed to run or not. But you
// MUST return an array instead of a logic value in order to foil "cracks"
// that use a debugging tool to read the return and then alter it. An array
// is a much harder return to read with a debugger than a logic value.
// The aBack array contains 10 numbers, only 4 of which are meaningful, and
// in your app, you read the array elements with those 4 numbers and convert
// them to their corresponding logic values and then make the app respond
// accordingly. It's more difficult for a hacker to crack this approach.

RETURN aBack            // End of ProPlus() main function

*-------------------------------------------------------------
*-------------------------------------------------------------
  // Called from Proplus() if app found to be in demo mode...
STATIC FUNCTION DemoFunc(nCrackChk,cDate)

LOCAL ;
      aRay      := {}        ,;
      cGama     := SPACE(5)  ,;
      cStand    := SPACE(15) ,;
      cGetcd    := SPACE(1)  ,;
      cTestm    := ""        ,;
      cDate8    := ""        ,;
      cExecleft := "  "      ,;
      cExpType  := SPACE(10) ,;
      cDlrfil   := ""        ,;
      cMsg1     := ""        ,;
      cMsg2     := ""        ,;
      cMsg3     := ""        ,;
      cMsg11    := ""        ,;
      cMsg12    := ""        ,;
      cMsg13    := ""        ,;
      lAns      := .f.       ,;
      lExeLimit := .f.       ,;
      lFwrd     := .f.       ,;
      nExecleft := 0         ,;
      nExpCount := 0         ,;
      nExpLimit := 0         ,;
      nlRet     := 0         ,;
      nTest     := 0         ,;
      nTestb    := 0         ,;
      nTestl    := 0         ,;
      nTsta     := 0         ,;
      oDlgA                  ,;
      oDlgE                  ,;
      oDlgOK                 ,;
      oDlgTF                 ,;
      oExecleft              ,;
      oMsg1                  ,;
      oMsg2                  ,;
      oMsg3                  ,;
      oMsg11                 ,;
      oMsg12                 ,;
      oMsg13                 ,;
      oPhone                 ,;
      Stopjp    := ""        ,;
      Stopmem   := ""

FIELD Gama, Stand, Diskpro, Tenfld
FIELD Dlrname, Dlrphone, Dlrfax, Dlremail, Dlrwebsite

    cMsg12 := "Unlock only available at server."

    // If the copy protection test passed, lDemo_ver will be .F., otherwise .T.

    // If we got to this spot, the copy protection test failed.

    pp_getvarchar( nLFhandle, VAR_EXPIRE_TYPE, @cExpType )
    cExpType := SUBSTR(cExpType,1,1)

    IF cExpType = "E"

       nTest := pp_getvarnum( nLFhandle, VAR_EXP_LIMIT, @nExpLimit )

       if nTest > 0 .AND. EMPTY( nExpLimit )
          pp_setvarchar( nLFhandle, VAR_EXPIRE_TYPE, EXP_EXE_LIMIT )
          pp_setvarnum( nLFhandle, VAR_EXP_LIMIT, 10 )
       endif

       SysRefresh()
       CursorWait()

                // get count of app executions made, from license file...
       nChk :=  pp_getvarnum(nLFhandle,VAR_EXP_COUNT,nTestcnt)

    ENDIF // cExpType

IF cExpType # "F"  // Don't use this section for function-limited demos...

    IF InitFile->( Netlock( RECORDLOCK ) )
      ELSE    // Netlock()
        Nonlock("InitFile")
        lDemo_ver := .t.
        aBack[4]  := nDemoTrue
        aBack[8]  := nExTrue
        RETURN aBack
    ENDIF     // Netlock()

*   InitFile.dbf:          // Gama is encrypted VAR_EXE_COUNT and
                           // Stand is encryption of last date that the app
                           // was launched.  These are used as cross-checks
                           // to foil attempts to defeat demo exec. limit.
                           // Stand date checked against last date JOURNAL.DBF
                           // was updated, which if more recent than Stand
                           // means InitFile.DBF was replaced with an earlier
                           // version to try to regain earlier executions
                           // allownace (which was higher).  If Gama and the
                           // license file's VAR_EXE_COUNT don't agree because
                           // user replaced license file with earlier version 
                           // to regain earlier count allowance, then Gama # 
                           // is used and license file's # corrected to match.

    cGama  := SPACE(5)
    cStand := SPACE(15)
    pp_decrypt(Gama,cPwstr,@cGama)
    nTestb := VAL(SUBSTR(cGama,1,2))
    pp_decrypt(Stand,cPwstr,@cStand)
    cTestm := SUBSTR(cStand,1,8)

    aRay := {}
    // Get latest modification date from a file that is heavily used by the
    // app, to use it as a crosscheck date of most recent app access... 
    IF nVersion = 1 .AND. FILE(rtrim(cPath) + "\Wrk\JOURNAL.DBF")
       aRay := Directory(rtrim(cPath) + "\Wrk\JOURNAL.DBF")
    ELSEIF nVersion = 2 .AND. FILE(rtrim(cPath) + "\Wrk\JOURNAL.DBF")
       aRay := Directory(rtrim(cPath) + "\Wrk\JOURNAL.DBF")
    ELSEIF nVersion = 3 .AND. FILE(rtrim(cPath) + "\Wrk\RESPLOG.DBF")
       aRay := Directory(rtrim(cPath) + "\Wrk\RESPLOG.DBF")
    ELSE
       IF nVersion > 3 .AND. FILE(rtrim(cPath) + "\RESET\JOURNAL.DBF")
          aRay := Directory(rtrim(cPath) + "\RESET\JOURNAL.DBF")
       ENDIF
    ENDIF

    IF EMPTY(aRay)
       MsgAlert("\Wrk\Journal.dbf file not found. Will cancel.")
       lDemo_ver := .t.
       aBack[4]  := nDemoTrue
       nCrackChk     := 4
       aBack[6]  := nCrackChk
       lExit       := .t.
       aBack[8]  := nExTrue
       pp_lfclose(nLFhandle)
       nLFhandle := 0
       WritePProString("STOPS","Stopq" , "F", cSemaphor ) 
       RETURN aBack
    ENDIF

    dTdate  := aRay[1,3]
    dTdatem := CTOD(cTestm)
    dTdate2 := Date() + 12

    // Test against InitFile crosscheck data if using exec. limit demo...
    IF cExpType = "E" .AND. nTestb # nTestcnt
       nTestl := Max(nTestb,nTestcnt)

       nChk := pp_setvarnum(nLFhandle,VAR_EXP_COUNT, nTestl )

       SysRefresh()
       CursorWait()

       IF nChk = 0
          cMsg12 := "Control file error: set at readonly."
          cMsg13 := "Call tech support " + cPhone
          MsgOk(cMsg12,cMsg13)
          lExit := .t.
          aBack[8]  := nExTrue
          CLOSE DATABASES
          RETURN aBack
       ENDIF
    ENDIF

    // dTdate from JOURNAL file date; dTdatem from InitFile Stand field...
    //                if file date > Stand date...
    IF cExpType = "E" .AND. dTdate > dTdatem    // for "E" only???
       nChk := pp_setvarnum(nLFhandle,VAR_EXP_LIMIT,10)

       SysRefresh()
       CursorWait()

       IF nChk = 1
          nChk := pp_setvarnum(nLFhandle,VAR_EXP_COUNT,9)
       ENDIF

       SysRefresh()
       CursorWait()

       IF nChk = 0
          cMsg12 := "Control file error: set at readonly."
          cMsg13 := "Call tech support " + cPhone
          MsgOk(cMsg12,cMsg13)
          lExit := .t.
          aBack[8]  := nExTrue
          CLOSE DATABASES
          RETURN aBack
       ENDIF

       cMsg1 := "Error reading control file B."
       cMsg2 := "Call tech support " + cPhone
       cMsg3 := "Are they on phone with you now?"

       lAns := MsgTF(cMsg1,cMsg2,cMsg3)

       IF lAns = .t.
          lFwrd := .f.
          aBack := Get_Code(0)
          nCrackChk := aBack[6]
       ENDIF
          lDemo_ver := .t.
          aBack[4]  := nDemoTrue
          nCrackChk     := 4
          aBack[6]  := nCrackChk
          lExit       := .t.
          aBack[8]  := nExTrue
       RETURN aBack
    ENDIF

    // Test if InitFile Stand given an advanced date > 12 days...
    IF dTdatem > dTdate2   
       nChk := pp_setvarnum(nLFhandle,VAR_EXP_LIMIT,10) 

       SysRefresh()
       CursorWait()

       IF nChk = 1
          nChk := pp_setvarnum(nLFhandle,VAR_EXP_COUNT,9)
       ENDIF

       SysRefresh()
       CursorWait()

       IF nChk = 0
          cMsg12 := "Control file error: set at readonly."
          cMsg13 := "Call tech support " + cPhone
          MsgOk(cMsg12,cMsg13)
          lExit := .t.
          aBack[8]  := nExTrue
          CLOSE DATABASES
          RETURN aBack
       ENDIF

       cMsg1 := "Error reading control file C."
       cMsg2 := "Call tech support" + cPhone
       cMsg3 := "Are they on phone with you now?"

       lAns := MsgTF(cMsg1,cMsg2,cMsg3)

       IF lAns = .t.
          lFwrd := .f.
          Get_Code(0)
       ENDIF
          lExit := .t.
          aBack[8]  := nExTrue
       RETURN aBack
    ENDIF

    cDate8 := SUBSTR(cDate,1,6)+SUBSTR(cDate,9,2)

        // (A workaround used here due to PLUS V3 wrapper problem)
    IF (cExpType = "E" .AND. pp_expired(nLFhandle) > 0 ) .OR. ;
       (cExpType = "D" .AND. DATE() > CTOD(cDate8) )

      IF cExpType = "D"
         pp_upddate(nLFhandle, UPDDATE_NOTIME )
         lUpdDate := .t.
      ENDIF

      DEFINE DIALOG oDlgE RESOURCE "STARTUP_F" OF oWnd

/*      The msg text below appears on this dialog, which replaces the dangerous 
        nonmodal message boxes that had allowed users to circumvent security.
        (If message boxes must be used, only use MODAL versions that are
         provided in some functions within this prg.)

        MsgAlert("Sorry, but the demo mode for"+CRLF+;
                "this program has expired."+CRLF+;
                "Please call and register with"+CRLF+;
                cCoName +CRLF+ "at " + cPhone,"Alert")
        MsgInfo("For more information about"+CRLF+;
                "the demo, or pricing, press 'Help.'")
*/
      REDEFINE BUTTON ID 1 OF oDlgE ;   // Okay
        ACTION ( lExit := .t. , aBack[8] := nExTrue, oDlgE:End() )

      REDEFINE BUTTON ID 6 OF oDlgE ;   // UNLOCK
        ACTION ( lFwrd := .f., aBack := Get_Code(0),;
                 nCrackChk := aBack[6],;
                 lExit := .t., aBack[8] := nExTrue,oDlgE:End() )  

      REDEFINE BUTTON ID 4 OF oDlgE ;   // Help
        ACTION HelpTopic(1099) 

      ACTIVATE DIALOG oDlgE CENTERED ;
        ON INIT (oDlgE:Move( nWtop+37, nWleft+50 ), SysWait(.2), ;
                 SetFocus(oDlgE)) RESIZE16

      lExit := .t.
      aBack[8]  := nExTrue

      RETURN aBack

    ENDIF

  //           *****     DEMO MODE SCREEN     ******

 nTest := pp_getvarnum( nLFhandle, VAR_EXP_LIMIT, @nExpLimit )

 IF nTest > 0 .AND. !empty( nExpLimit ) .AND. cExpType = "E"
    pp_getvarnum(nLFhandle,VAR_EXP_LIMIT,@nExpLimit )
    pp_getvarnum(nLFhandle,VAR_EXP_COUNT,@nExpCount )
    nExecleft = nExpLimit - nExpCount    // returned by reference.
    cExecleft := STR(nExecleft,2)

    lFwrd := .t.   // ???

    SysWait(.2)

   DEFINE DIALOG oDlgA RESOURCE "STARTUP_A" OF oWnd

   REDEFINE GET oExecleft VAR cExecleft ID 21 OF oDlgA COLOR CLR_BLACK 
   REDEFINE SAY oPhone ID 22 PROMPT cPhone OF oDlgA COLORS CLR_BLACK, CLR_HGRAY

   REDEFINE BUTTON ID 1 OF oDlgA ;   // Okay
     ACTION ( lFwrd := .t. , oDlgA:End() )

     // End screen, don't update record on way out.
   REDEFINE BUTTON ID 2 OF oDlgA ;   // Cancel
     ACTION ( lFwrd := .f., oDlgA:End(), lExit := .t., aBack[8] := nExTrue )  

   REDEFINE BUTTON ID 6 OF oDlgA ;   // Unlock
     ACTION ( lFwrd := .f., aBack := Get_Code(@nExecleft), ;
              nCrackChk := aBack[6],oExecleft:Refresh(),;
              lExit := .t., aBack[8] := nExTrue, oDlgA:SetFocus() )  

   REDEFINE BUTTON ID 4 OF oDlgA ;   // Help
     ACTION HelpTopic(1099) 

   ACTIVATE DIALOG oDlgA CENTERED ;
      ON INIT (oDlgA:Move( nWtop+37, nWleft+50 ),;
               oExecleft:Refresh(),;
               SysWait(.2), SetFocus(oDlgA)) RESIZE16

 ENDIF // nTest

 IF lFwrd = .t.

   nlRet := pp_getvarchar(nLFhandle, VAR_EXPIRE_TYPE, @cExpType)
   cExpType := SUBSTR(cExpType,1,1)

   IF nlRet = 1 .AND. cExpType == "E"
      lExelimit := .t.
     ELSE
      lExelimit := .f.
   ENDIF

   IF lExelimit
               // Increment the program execution counter in the control file.
     nChk := pp_countinc( nLFhandle, VAR_EXP_COUNT)


     IF nChk = 0
          cMsg12 := "Control file error: set at readonly."
          cMsg13 := "Call tech support " + cPhone
          MsgOk(cMsg12,cMsg13)
          lExit := .t.
          aBack[8]  := nExTrue
          CLOSE DATABASES
          RETURN aBack
     ENDIF

     IF InitFile->( Netlock( RECORDLOCK ) )
       ELSE    // Netlock()
        Nonlock("Wrk\InitFile")
        RETURN aBack
     ENDIF     // Netlock()

     cNewGama := SPACE(5)

     pp_getvarnum(nLFhandle,VAR_EXP_COUNT,@nExpCount )
     pp_encrypt(STR(nExpCount,2),cPwstr,@cNewGama)
     cNewGama := SUBSTR(cNewGama,1,2)

     REPLACE Gama WITH cNewGama

   ENDIF // lExelimit

     // Protect Stand from change if innocent short-term date jockeying...
   IF dTdatem < Date() .OR. dTdatem > dTdate2   

      cStand := SPACE(15) 
      pp_encrypt(DTOC(date()),cPwstr,@cStand)
      cStand := SUBSTR(cStand,1,8)

      REPLACE Stand WITH cStand

   ENDIF

 ENDIF  // lFwrd

 InitFile->( dbCommit() )
 InitFile->( dbUnLock() )

ENDIF // if cExpType # "F"

aBack[6] := nCrackChk

RETURN aBack
*gc--------------------------------------------
  // Single user unlock function called from app directly, from Proplus() or
  // from DemoFunc().   (Must be done before authorizing lan count.)...
FUNCTION Get_Code(nExecleft)
         // meanwhile have record lock on InitFile

    LOCAL ;
          aStats   := {}        ,;
          cCode_no := ""        ,;
          cFileNo  := ""        ,;
          cMsg12   := ""        ,;
          cSerial  := SPACE(45) ,;
          cSerial2 := " "       ,;
          cInitFil := ""        ,;
          Getlist  := {}        ,;
          lOkay    := .f.       ,;
          nCompno  := 0         ,;
          nDiskpr  := 0         ,;
          nGet_code := 0        ,;
          nLast    := 0         ,;
          nSfcnt   := 0         ,;
          nSerial  := 0         ,;
          nTest    := 0         ,;
          oCodeno               ,;
          oDlgB                 ,;
          oDlgOk                ,;
          oGetcode              ,;
          oMsg12                ,;
          oName                 ,;
          oSerial 

    STATIC nCrackChk := 4

    FIELD Stand,Gama IN InitFile

    aStats    := GetAppStats()
    oWnd      := aStats[1]
    nWtop     := aStats[2]
    nWleft    := aStats[3]
    cPath     := aStats[4]

    aBack := {34,83,57,23,0,71,48,51,95,69}
    aBack[6] := nCrackChk

    IF nVersion = 3
       cInitFil := RTRIM(cPath) + "\Wrk\InitFil2"
    ELSEIF nVersion = 1
       cInitFil := RTRIM(cPath) + "\Wrk\InitFile"
    ELSEIF nVersion = 2
       cInitFil := RTRIM(cPath) + "\Wrk\InitFile"
    ELSE
       cInitFil := RTRIM(cPath) + "\RESET\InitFile"
    ENDIF
    
    IF SELECT("InitFile") = 0
       IF_NETOPEN (cInitFil) ALIAS InitFile SHARED
          // Check if InitFile has Diskpro field; if not, add it with disk #.
          nSfcnt := FCOUNT()
          IF nSfcnt < 32       // Check if Wrk InitFile file has current layout.
             SETCOPY(nSfcnt)   // If not, then rebuild file with added fields
          ENDIF
       ELSE
          Nonopen("Wrk\InitFile")
          RETURN aBack
       ENDIF
    ENDIF
    IF InitFile->( Netlock( RECORDLOCK ) )
       ELSE    // Netlock()
         Nonlock("RESET\InitFile")
         aBack[6] := nCrackChk
         RETURN aBack
    ENDIF     // Netlock()

    //                     *****  APP UNLOCKING ROUTINE    *****

    nDiskpr := 0
    
     nCode_no = InitFile->Testnum
    IF EMPTY(nCode_no)
      nCode_no = pp_cenum()
      SysRefresh()
      REPLACE Testnum WITH nCode_no
    ENDIF

    cCode_no := ltrim(str(nCode_no))

    // Get product serial number...
    pp_getvarnum(nLFhandle, VAR_SERIAL_NUM, @nSerial)
    cSerial := RTRIM(STR(nSerial))

    // The following two lines eliminate the "end-of-string" char ( ASCII 0 )
    // which would cause the second half of a string concatenation to disappear.
*   nLast := LEN(RTRIM(cSerial))
*   cSerial := SUBSTR(cSerial,1,nLast-1)

    //The pp_compno function gives you a computer ID number.

    IF lLocalDr  // Used for either single user or the server in a network...
       nCompno := pp_compno( COMPNO_ENHANCED, "", "" )
    ELSE  // Using type of return value usually called nNetname as nCompno.
          // Here we are unlocking one workstation in order to get "nNetname"
          // into the list of authorized machines, to serve for all workstatns.
       nCompno := pp_compno(COMPNO_NETNAME,"",cDrive)
    ENDIF

    nGet_code := 0

*    cSerial2 := cSerial + " / " + LTRIM( STR( nCompno ) ) // Old way, but the
                                                           // string is too
                                                           // long for display.
    cSerial2 := LTRIM( STR( nCompno ) )

   DEFINE DIALOG oDlgB RESOURCE "STARTUP_B" OF oWnd

   // "Source:" (distributor's name)
   REDEFINE SAY oName    ID 23 PROMPT cCoName OF oDlgB

   // "Computer reference number:"
   REDEFINE SAY oSerial  ID 22 PROMPT cSerial2 OF oDlgB

   // "Number to give customer service:"
   REDEFINE SAY oCodeno  ID 24 PROMPT cCode_no OF oDlgB

   // "Enter the code number that they give to you over the phone:"
   REDEFINE GET oGetcode VAR nGet_code picture "@BZ" ID 110 OF oDlgB
        oGetcode:bGotFocus = {||oGetcode:SetPos(0,0)}

   REDEFINE BUTTON ID 1 OF oDlgB ;   // Okay
     ACTION ( lOkay := .f. , ;
              nCrackChk := CodeProc( @nExecleft, nCode_no, nCompno,;
                                             nGet_code, nCrackChk, @lOkay, oDlgB ),;
              aBack[6] := nCrackChk,;
              IF(lOkay, oDlgB:End(),) ;
            )

        // End screen, don't update record on way out.
   REDEFINE BUTTON ID 2 OF oDlgB ;   // Cancel
     ACTION ( lOkay := .f., oDlgB:End() )  

        // Help.
   REDEFINE BUTTON ID 4 OF oDlgB ;   // Help
     ACTION ( HelpTopic(7020) )  

   ACTIVATE DIALOG oDlgB CENTERED ;
      ON INIT (oDlgB:Move( nWtop+37, nWleft+50 ), SysWait(.2), SetFocus(oDlgB)) RESIZE16

    InitFile->( dbCommit() )
    InitFile->( dbUnLock() )

   aBack[6] := nCrackChk  

RETURN aBack
*---------------------------------------------------------------------------
  // Unlock app or extend # of demo executions or days available...
STATIC FUNCTION CodeProc( nExecleft, nCode_no, nCompno, nGet_code, ;
                          nCrackChk, lOkay, oDlgB )
   LOCAL ;
         cCode_no  := ""         ,;
         cExpDate  := ""         ,;
         cExpType  := SPACE(10)  ,;
         cFileNo   := ""         ,;
         cStand    := SPACE(15)  ,;
         cMsg12    := ""         ,;
         cMsg13    := ""         ,;
         cSerial2  := " "        ,;
         cSerial   := SPACE(45)  ,;
         cTenf     := SPACE(15)  ,;
         cTenfld   := ""         ,;
         dExpDate  := CTOD("")   ,;
         Getlist   := {}         ,;
         nDD       := 0          ,;
         nExpCount := 0          ,;
         nExpLimit := 0          ,;
         nLast     := 0          ,;
         nMM       := 0          ,;
         nRet      := 0          ,;
         nRoutine  := 0          ,;
         nSfcnt    := 0          ,;
         nSuccess  := 0          ,;
         nTest     := 0          ,;
         nYYYY     := 0          ,;
         oDlgOk                  ,;
         oMsg12                  ,;
         oMsg13

   FIELD Stand,Gama IN InitFile

   cSemaphor := CurDrive() + ":\" +CurDir()+ "\Semaphor.ini"

   // Get demo expiration type...
   nRet := pp_getvarchar(nLFhandle,VAR_EXPIRE_TYPE,@cExpType)

   IF nRet = 1
      cExpType := SUBSTR(cExpType,1,1)
   ELSE
      cExpType := "E"
   ENDIF

   nRoutine = pp_tcode( nGet_code, nCode_no, nCompno, nSeed )

   // Yes, there seems to be a lot of needless repetition among the case
   // statements here that might be eliminated by calls to a single
   // function for each repetetive codeset. However, must analyze this very
   // carefully first to make sure that doing so won't jeopardize security
   // in any way...

   DO CASE
        CASE nRoutine = 2  // reset last app launch date in lic. file

                      // Force last-launch-date to current system date, but
                      // without use of time (because LAN workstations may not
                      // have synchronized internal clocks).
             nTest := pp_upddate( nLFhandle, UPDDATE_FORCE+UPDDATE_NOTIME )
             lUpdDate := .t.

             IF nTest = PP_SUCCESS

                SELECT InitFile
                cStand := SPACE(15) 
                pp_encrypt(DTOC(date()),cPwstr,@cStand)
                cStand := SUBSTR(cStand,1,8)

                REPLACE Stand WITH cStand

                cMsg12 := "Last-launch-date error corrected."
                cMsg13 := ""

                lGoOn := .t.
             ENDIF

        CASE nRoutine = 3  // Unlock app on a single computer. 

                // Add current computer to list of unlocked machines...
             IF lLocalDr
                nTest := pp_copyadd( nLFhandle, COPYADD_ENHANCED, 0 )
             ELSE  // for network workstation to add nNetname (as nCompno)...
                nTest := pp_copyadd( nLFhandle, COPYADD_ADDDUPE, nCompno )
             ENDIF
                // Using the above on a network, we will record one unlocked
                // computer number for the server (COMPNO_ENHANCED type of no)
                // and one more unlocked computer number for all other
                // workstations on the network (COMPNO_NETNAME type of no).
                // Then, when a user launches the app, PLUS will check to see
                // if the machine offers one of these two numbers in order
                // to be initially authorized to continue the launch. Next,
                // on a network, PLUS checks to see if enough floating net
                // user rights are available to allow the user to have access.

             IF nTest = PP_SUCCESS

                REPLACE Testnum WITH 0
                lDemo_ver := .f.
                aBack[4]  := nDemFalse
                nCrackChk := 83
                aBack[6]  := nCrackChk

                 // allows use from a LAN workstation...
                nTest := pp_setvarnum(nLFhandle, VAR_LAN_LIMIT, 1 )

                SysRefresh()
                CursorWait()

                cMsg12 := "Unlimited program operation"
                cMsg13 := "authorized."

                lGoOn := .t.
             ENDIF

        CASE nRoutine = 5   // reset to allow 5 more demo executions/days...
           IF cExpType = "E"     // Executions limit type of demo...
             IF pp_setvarnum(nLFhandle, VAR_EXP_COUNT, 0 ) = 1 .AND. ;
                pp_setvarnum(nLFhandle, VAR_EXP_LIMIT, 5 ) = 1

                SysRefresh()
                CursorWait()

                // Get new expiration count, encrypt it and save it
                // redundantly in InitFile table as a cross-check value...
                cNewGama := SPACE(5)
                pp_getvarnum(nLFhandle,VAR_EXP_COUNT,@nExpCount )
                pp_encrypt(STR(nExpCount,2),cPwstr,@cNewGama)
                cNewGama := SUBSTR(cNewGama,1,2)
                REPLACE Gama WITH cNewGama
                cMsg12 := "Program demo executions limit reset"
                cMsg13 := "to 5 more."
                lGoOn := .t.
             ENDIF // pp_setvarnum()
           ELSEIF cExpType = "D" // Date expiration type of demo...
                SET CENTURY ON
                pp_getvardate(nLFhandle,VAR_EXP_DATE_SOFT,@nMM,@nDD,@nYYYY)
                cExpDate := STR(nMM,2)+"/"+STR(nDD,2)+"/"+STR(nYYYY,4)
                dExpDate := CTOD(STRTRAN(cExpDate," ","0"))
                dExpDate := dExpDate + 5
                cExpDate := DTOC(dExpDate)
                nMM := VAL(SUBSTR(cExpDate,1,2))
                nDD := VAL(SUBSTR(cExpDate,4,2))
                nYYYY := VAL(SUBSTR(cExpDate,7,4))
                pp_setvardate(nLFhandle,VAR_EXP_DATE_SOFT,nMM,nDD,nYYYY)

                SysRefresh()
                CursorWait()

                // Encrypt new exp date and save redundantly for cross-check...
                SELECT InitFile
                cTenf := SPACE(15)
                pp_encrypt(cExpDate, cPwstr2, @cTenf)
                cTenfld := SUBSTR(cTenf,1,10)
                SET CENTURY OFF
                REPLACE Tenfld WITH cTenfld
                cMsg12 := "Program demo expiration date increased"
                cMsg13 := "by 5 days."
                lGoOn := .t.
           ELSE  // Other cExpType...
                cMsg12 := "Using other expiration type: "
                cMsg13 := cExpType
           ENDIF // cExpType

        CASE nRoutine = 10   // reset to allow 10 more demo executions/days...
           IF cExpType = "E"     // Executions limit type of demo...
             IF pp_setvarnum(nLFhandle, VAR_EXP_COUNT, 0 ) = 1 .AND. ;
               pp_setvarnum(nLFhandle, VAR_EXP_LIMIT, 10 ) = 1

                SysRefresh()
                CursorWait()

                IF dTdatem < Date() .OR. dTdatem > dTdate2 
                                 // Protects Stand from change if 
                                 // innocent short-term date jockeying
                   cStand := SPACE(15) 
                   pp_encrypt(DTOC(date()),cPwstr,@cStand)
                   cStand := SUBSTR(cStand,1,8)
                   REPLACE Stand WITH cStand
                ENDIF
                cNewGama := SPACE(5)

                pp_getvarnum(nLFhandle,VAR_EXP_COUNT,@nExpCount )
                pp_encrypt(STR(nExpCount,2),cPwstr,@cNewGama)
                cNewGama := SUBSTR(cNewGama,1,2)

                REPLACE Gama WITH cNewGama
                REPLACE Testnum WITH 0

                SysRefresh()
                CursorWait()

                cMsg12 := "Program demo executions limit reset"
                cMsg13 := "to 10 more."
                lGoOn := .t.

             ENDIF // pp_setvarnum()
           ELSEIF cExpType = "D" // Date expiration type of demo...
                SET CENTURY ON
                pp_getvardate(nLFhandle,VAR_EXP_DATE_SOFT,@nMM,@nDD,@nYYYY)
                cExpDate := STR(nMM,2)+"/"+STR(nDD,2)+"/"+STR(nYYYY,4)
                dExpDate := CTOD(STRTRAN(cExpDate," ","0"))
                dExpDate := dExpDate + 10
                cExpDate := DTOC(dExpDate)
                nMM := VAL(SUBSTR(cExpDate,1,2))
                nDD := VAL(SUBSTR(cExpDate,4,2))
                nYYYY := VAL(SUBSTR(cExpDate,7,4))
                pp_setvardate(nLFhandle,VAR_EXP_DATE_SOFT,nMM,nDD,nYYYY)

                SysRefresh()
                CursorWait()

                // Encrypt new exp date and save redundantly for cross-check...
                SELECT InitFile
                cTenf := SPACE(15)
                pp_encrypt(cExpDate, cPwstr2, @cTenf)
                cTenfld := SUBSTR(cTenf,1,10)
                SET CENTURY OFF
                REPLACE Tenfld WITH cTenfld
                cMsg12 := "Program demo expiration date increased"
                cMsg13 := "by 10 days."
                lGoOn := .t.
           ELSE  // Other cExpType...
                cMsg12 := "Using other expiration type: "
                cMsg13 := cExpType
           ENDIF // cExpType

        CASE nRoutine = 12   // reset to allow 20 more demo executions/days...
           IF cExpType = "E"     // Executions limit type of demo...
             IF pp_setvarnum(nLFhandle, VAR_EXP_COUNT, 0 ) = 1 .AND. ;
               pp_setvarnum(nLFhandle, VAR_EXP_LIMIT, 20 ) = 1

                SysRefresh()
                CursorWait()

                 // Why this here ??  Left over from V3 ??
*                nSuccess := pp_lfopen( cFile, LFOPEN_NOCACHE, LF_FILE, cPword , nLFhandle)

                SysRefresh()
                CursorWait()

                IF dTdatem < Date() .OR. dTdatem > dTdate2 
                                 // Protects Stand from change if 
                                 // innocent short-term date jockeying
                   cStand := SPACE(15) 
                   pp_encrypt(DTOC(date()),cPwstr,@cStand)
                   cStand := SUBSTR(cStand,1,8)
                   REPLACE Stand WITH cStand
                ENDIF
                cNewGama := SPACE(5)

                pp_getvarnum(nLFhandle,VAR_EXP_COUNT,@nExpCount )
                pp_encrypt(STR(nExpCount,2),cPwstr,@cNewGama)
                cNewGama := SUBSTR(cNewGama,1,2)

                REPLACE Gama WITH cNewGama
                REPLACE Testnum WITH 0

                SysRefresh()
                CursorWait()

                cMsg12 := "Program demo executions limit reset"
                cMsg13 := "to 20 more."
                lGoOn := .t.

             ENDIF // pp_setvarnum()
           ELSEIF cExpType = "D" // Date expiration type of demo...
                SET CENTURY ON
                pp_getvardate(nLFhandle,VAR_EXP_DATE_SOFT,@nMM,@nDD,@nYYYY)
                cExpDate := STR(nMM,2)+"/"+STR(nDD,2)+"/"+STR(nYYYY,4)
                dExpDate := CTOD(STRTRAN(cExpDate," ","0"))
                dExpDate := dExpDate + 20
                cExpDate := DTOC(dExpDate)
                nMM := VAL(SUBSTR(cExpDate,1,2))
                nDD := VAL(SUBSTR(cExpDate,4,2))
                nYYYY := VAL(SUBSTR(cExpDate,7,4))
                pp_setvardate(nLFhandle,VAR_EXP_DATE_SOFT,nMM,nDD,nYYYY)

                SysRefresh()
                CursorWait()

                // Encrypt new exp date and save redundantly for cross-check...
                SELECT InitFile
                cTenf := SPACE(15)
                pp_encrypt(cExpDate, cPwstr2, @cTenf)
                cTenfld := SUBSTR(cTenf,1,10)
                SET CENTURY OFF
                REPLACE Tenfld WITH cTenfld

                SysRefresh()
                CursorWait()

                cMsg12 := "Program demo expiration date increased"
                cMsg13 := "by 20 days."
                lGoOn := .t.
           ELSE  // Other cExpType...
                cMsg12 := "Using other expiration type: "
                cMsg13 := cExpType
           ENDIF // cExpType

        CASE nRoutine = 15   // reset to allow 30 more demo executions/days...
           IF cExpType = "E"     // Executions limit type of demo...
             IF pp_setvarnum(nLFhandle, VAR_EXP_COUNT, 0 ) = 1 .AND. ;
               pp_setvarnum(nLFhandle, VAR_EXP_LIMIT, 30 ) = 1

                SysRefresh()
                CursorWait()

                IF dTdatem < Date() .OR. dTdatem > dTdate2
                                 // Protects Stand from change if 
                                 // innocent short-term date jockeying
                   cStand := SPACE(15) 
                   pp_encrypt(DTOC(date()),cPwstr,@cStand)
                   cStand := SUBSTR(cStand,1,8)
                   REPLACE Stand WITH cStand
                ENDIF

                SysRefresh()
                CursorWait()

                cNewGama := SPACE(5)

                pp_getvarnum(nLFhandle,VAR_EXP_COUNT,@nExpCount )
                pp_encrypt(STR(nExpCount,2),cPwstr,@cNewGama)
                cNewGama := SUBSTR(cNewGama,1,2)

                Replace Gama WITH cNewGama
                REPLACE Testnum WITH 0

                cMsg12 := "Program demo executions limit reset"
                cMsg13 := "to 30 more."
                lGoOn := .t.

             ENDIF // pp_setvarnum()

           ELSEIF cExpType = "D" // Date expiration type of demo...
                SET CENTURY ON
                pp_getvardate(nLFhandle,VAR_EXP_DATE_SOFT,@nMM,@nDD,@nYYYY)
                cExpDate := STR(nMM,2)+"/"+STR(nDD,2)+"/"+STR(nYYYY,4)
                dExpDate := CTOD(STRTRAN(cExpDate," ","0"))
                dExpDate := dExpDate + 30
                cExpDate := DTOC(dExpDate)
                nMM := VAL(SUBSTR(cExpDate,1,2))
                nDD := VAL(SUBSTR(cExpDate,4,2))
                nYYYY := VAL(SUBSTR(cExpDate,7,4))
                pp_setvardate(nLFhandle,VAR_EXP_DATE_SOFT,nMM,nDD,nYYYY)

                SysRefresh()
                CursorWait()

                // Encrypt new exp date and save redundantly for cross-check...
                SELECT InitFile
                cTenf := SPACE(15)
                pp_encrypt(cExpDate, cPwstr2, @cTenf)
                cTenfld := SUBSTR(cTenf,1,10)
                SET CENTURY OFF
                REPLACE Tenfld WITH cTenfld

                SysRefresh()
                CursorWait()

                cMsg12 := "Program demo expiration date increased"
                cMsg13 := "by 30 days."
                lGoOn := .t.
           ELSE  // Other cExpType...
                cMsg12 := "Using other expiration type: "
                cMsg13 := cExpType
           ENDIF // cExpType

        CASE nRoutine = 0                         // error

           cMsg12 := "Incorrect code entered."
           cMsg13 := "."
           lOkay := .f.

        OTHERWISE                               // anything else

           cMsg12 := "Routine # to be executed = " + alltrim(str( nRoutine ))
           cMsg13 := "."

           lOkay := .f.

    ENDCASE

    // Protect Stand from chg if innocent short-term date jockeying
    IF lGoOn
       IF dTdatem < Date() .OR. dTdatem > dTdate2  
          cStand := SPACE(15) 
          pp_encrypt(DTOC(date()),cPwstr,@cStand)
          cStand := SUBSTR(cStand,1,8)
          REPLACE Stand WITH cStand
       ENDIF
       REPLACE Testnum WITH 0

       SysRefresh()
       CursorWait()

       InitFile->( dbCommit() )
       InitFile->( dbUnLock() )
    ENDIF

      // Display result message...
    DEFINE DIALOG oDlgOK RESOURCE "MsgOK" of oDlgB
    REDEFINE SAY oMsg12 PROMPT cMsg12 ID 22 OF oDlgOK
    REDEFINE SAY oMsg13 PROMPT cMsg13 ID 23 OF oDlgOK
    REDEFINE BUTTON ID 11 OF oDlgOK ;   // Yes
      ACTION ( oDlgOK:End() )
    ACTIVATE DIALOG oDlgOK CENTERED ;
      ON INIT (SetFocus(oDlgOK)) RESIZE16

       // It is important to call the following code to log this
       // workstation out before EVERY exit of the program.
       // But probably not appropriate to use here, because if we're here,
       // LAN usage probably hasn't been turned on yet. So it's here, but
       // inactivated, just in case we find that we need it later.
*    pp_getvarnum(nLFhandle,VAR_LAN_COUNT,@nCount)
*    IF pp_lanactive() = 1 .AND. nCount > 0
*       pp_countdec( nLFhandle, 1 )  //  <--- ***  to errorlog also!
*    ENDIF

    WritePProString("STOPS","Stopq" , "F", cSemaphor )     

    pp_getvarnum(nLFhandle,VAR_EXP_LIMIT,@nExpLimit )
    pp_getvarnum(nLFhandle,VAR_EXP_COUNT,@nExpCount )
    nExecleft = nExpLimit - nExpCount    // returned by reference.

RETURN nCrackChk
*--------------------------
// Make sure VAR_LAST_DATE filed in LF and also Stand field in aux file get 
// current date and that InitFile filedate will also change to today. 
// This is needed in order to automate ClearUsers() on first app startup 
// of each day...
STATIC PROCEDURE SetStand()
LOCAL ;
      cStand    := SPACE(15),;
      cInitFil  := ""     

  IF !lUpdDate  // Don't duplicate date update if already done...
     pp_upddate(nLFhandle, UPDDATE_NOTIME ) // Set LF w/ last date used (2day),
  ENDIF                                     // if not already done.

  // Redundant use of auxiliary file to record current app usage date...
  IF SELECT("InitFile") > 0
     CLOSE InitFile
  ENDIF

  IF nVersion = 3
     cInitFil := RTRIM(cPath) + "\Wrk\InitFil2"
    ELSEIF nVersion = 1
     cInitFil := RTRIM(cPath) + "\Wrk\InitFile"
    ELSEIF nVersion = 2
     cInitFil := RTRIM(cPath) + "\Wrk\InitFile"
    ELSE
     cInitFil := RTRIM(cPath) + "\RESET\InitFile"
  ENDIF

  IF SELECT("InitFile") = 0
   IF_NETOPEN (cInitFil) ALIAS InitFile SHARED
     ELSE
       Nonopen(IF(nVersion = 3,"Wrk\InitFil2","Wrk\InitFile"))
       RETURN
   ENDIF
  ENDIF
  SELECT InitFile

  SET CENTURY OFF
  pp_encrypt(DTOC(date()),cPwstr,@cStand)
  cStand  := SUBSTR(cStand,1,8)
  IF InitFile->( NetLock( RECORDLOCK ) ) 
     REPLACE Stand WITH cStand   
    ELSE
       Nonlock("InitFile - Stand")
  ENDIF
  InitFile->( dbCommit() )
  InitFile->( dbUnLock() )

RETURN
*gl--------------------------------------------------
  // Network authorization for # of simultaneous users, called from app...
FUNCTION Get_LanC()

  LOCAL ;
        aBackLan  := {}  ,;
        aStats    := {}  ,;
        cCenum    := ""  ,;
        cCompno   := ""  ,;
        cMessage  := "." ,;
        Getlist   := {}  ,;  // Retain Getlist for FiveWin here.
        lNewOpen  := .f. ,;
        lOkay     := .f. ,;
        nLimit    := 0   ,;
        nLimit1   := 0   ,;
        nOnline   := 0   ,;
        nOnline1  := 0   ,;
        nCe_num   := 0   ,;
        nCompno   := 0   ,;
        nCount    := 0   ,;
        nRoutine  := 0   ,;
        nGet_code := 0   ,;
        oCodeno          ,;
        oCompno          ,;
        oDlgC            ,;
        oGetcode         ,;
        oLimit           ,;
        oMessage         ,;
        oOnline          ,;
        oPhone

  aStats := GetAppStats()
  oWnd   := aStats[1]
  nWtop  := aStats[2]
  nWleft := aStats[3]
  cPath  := aStats[4]

  aBack  := {34,83,57,23,0,71,48,51,95,69}

  // nVersion is a filewide static.

  // nVersion tells which app called Proplus()...
  IF cLFile = "ActionL"
     nVersion := 1
  ELSEIF cLFile = "Surv360"
     nVersion := 2
  ELSE
     nVersion := 3
  ENDIF

  cFile := cPath+"\"+cLFile+".lf"     

     // Open Lic. File if not already open...
  IF EMPTY(nLFhandle)
     IF ( pp_lfopen( cFile, LFOPEN_NOCACHE, LF_FILE, cPword , @nLFhandle) ) = PP_FAILURE
        MsgAlert("Control file not available. Will cancel.")
        RETURN .f.
     ENDIF
     lNewOpen := .t.
  ENDIF     
  pp_getvarnum(nLFhandle, VAR_LAN_LIMIT, @nLimit)
  nLimit1 := nLimit

  pp_getvarnum(nLFhandle, VAR_LAN_COUNT, @nCount)
  nOnline1 := nOnline := nCount

  nCe_num   := pp_cenum()                  // create random #
  cCenum    := Ltrim(STR(nCe_num))
  nGet_code := 0

/*
  IMPORTANT: in a peer-to-peer network, when pp_compno(COMPNO_NETNAME, etc.) is
  used to get the numeric code representing the network's "name," you will get
  different results when doing this on the net's server computer as compared
  with doing this on a workstation computer. You will only get the correct
  number back in the latter case, when calling for this number from a workstatn
  computer.  This is because on the server, the drive letter that is used in
  the pp_compno() function is NOT a network drive, it is the server's own local
  C drive instead.  The correct result can only be gotten when querying a 
  network drive - thus only from a workstation. That is why setting up PLUS to
  implement floating licenses on a peer-to-peer LAN requires three unlocking
  procedures: 1) unlock the server as if it were a single user machine;
              2) unlock a workstation as if it were a single user machine, but
                 in this case the COMPNO_NETNAME parameter is used so that this
                 unlock is really for use by the whole network;
              3) unlock authorization for the number of floating LAN licences
                 that you want to authorize to enable concurrent workstation
                 users.

  Thereafter, when a workstation launches the app, PLUS will first check to
  see if the network is authorized to use the software (via COMPNO_NETNAME),
  and then will check to see if a floating license count is available for the
  requested concurrent sign-on. If the answers to these checks are yes-yes, 
  then the workstation can sign-on to the app.  In the case of the server
  trying to sign-on, the process is almost the same except that the first
  check uses COMPNO_ENHANCED because the authorization query is being made with
  regard to the local C drive instead of regarding a redirected network drive.

  Puzzling? Just remember the famous words of the 1950s movie cartoon character
  Elmer Fudd: "This is vewy vewy twicky!" - and you'll do just fine.

*/

  nCompno   := pp_compno(COMPNO_NETNAME,"",cDrive)
  IF nCompno = 0  // In rare case where NETNAME read on C drive machine is 0...
     nCompno   := pp_compno(COMPNO_ENHANCED,"","")  // use this instead.
  ENDIF
  cCompno   := ALLTRIM(STR(nCompno))


   DEFINE DIALOG oDlgC RESOURCE "STARTUP_C" OF oWnd

   // "Code A to give customer service:"
   REDEFINE SAY oCodeno  ID 22 PROMPT cCenum OF oDlgC UPDATE // display random #

   // "Code B to give customer service:"
   REDEFINE SAY oCompno  ID 32 PROMPT cCompno OF oDlgC UPDATE // display computer ID #

   // "(Service phone: 212-123-4567)"
   REDEFINE SAY oPhone   ID 23 PROMPT cPhone OF oDlgC

   // "Enter the code number given to you over the phone:"
   REDEFINE GET oGetcode VAR nGet_code picture "@Z" ID 110 OF oDlgC UPDATE

   // Message explaining result of code entered:
   REDEFINE SAY oMessage ID 24 PROMPT cMessage OF oDlgC UPDATE

   // "Simultaneous users limit:"
   REDEFINE SAY oLimit   ID 25 PROMPT nLimit   OF oDlgC UPDATE

   // "Users now on line:"
   REDEFINE SAY oOnline  ID 26 PROMPT nOnline  OF oDlgC UPDATE

   REDEFINE BUTTON ID 11 OF oDlgC ;   // Okay
     ACTION ( lOkay := .t., aBackLan := LanLimit(nGet_Code, nCe_num, nCompno),; 
              nLimit   := aBackLan[1],;
              cMessage := aBackLan[2],;
              nOnline  := aBackLan[3],;
              nCe_num  := pp_cenum(),;
              cCenum   := Ltrim(STR(nCe_num)),;
              nGet_code := 0,;
              oDlgC:Update()  )

     // End screen, don't update record on way out.
   REDEFINE BUTTON ID 2 OF oDlgC ;   // Cancel
     ACTION ( lOkay := .f., oDlgC:End() )  

     // Reset if user count wrong due to crash...
   REDEFINE BUTTON ID 5 OF oDlgC ; // Only works if just 1 user signed on.
     ACTION ( lOkay := .t. , ClearUsers(), oDlgC:End()  )

   REDEFINE BUTTON ID 4 OF oDlgC ;   // Help
     ACTION ( HelpTopic(7018) )  

   ACTIVATE DIALOG oDlgC CENTERED ;
      ON INIT (oDlgC:Move( nWtop+37, nWleft+50 ), SysWait(.2), SetFocus(oDlgC)) RESIZE16

   IF nLimit # nLimit1 .OR. nOnline > nLimit
      // it is important to call this function to log this workstation out
      // before EVERY exit of the program.  We will only do it here if we 
      // actually incremented it in the beginning of the file.  The variable
      // lDemo_ver is set to .F. if we incremented it.

      pp_getvarnum(nLFhandle,VAR_LAN_COUNT,@nCount)
      IF pp_lanactive() = 1 .AND. nCount > 0
         pp_countdec( nLFhandle, 1 )  //  <--- ***  to errorlog also!
      ENDIF
      lExit := .t.
      aBack[8]  := nExTrue
*     Exit()
   ENDIF

   IF lNewOpen .AND. !EMPTY(nLFhandle)
      pp_lfclose(nLFhandle)
      nLFhandle := 0
   ENDIF

RETURN aBack

*ll----------------------------------------
*lan
  // Processing of Get_LanC()choice..
STATIC FUNCTION LanLimit(nGet_Code, nCe_num, nCompno)

LOCAL ;
      aBackLan  := {}  ,;
      cAppCode  := ""  ,;
      cIniFile  := ""  ,;
      cIniFileW := ""  ,;
      cIniPath  := ""  ,;
      cSemDir   := ""  ,;
      cSemPath  := ""  ,;
      cMessage  := " " ,;
      cInitFil  := ""  ,;
      nCount    := 0   ,;
      nExpcount := 0   ,;
      nLimit    := 0   ,;
      nOnline   := 0   ,;
      nRet      := 0   ,;
      nRoutine  := 0   ,;
      nTGama    := 0

FIELD Gama,Testnum IN InitFile


cSemPath := cDrive+":\"+CurDir()
cAppCode := SUBSTR(cSemPath,RAT("\",cSemPath)+1,1)
cAppCode := cAppCode + SUBSTR(cSemPath,LEN(cSemPath),1)
cSemDir  := SUBSTR(cSemPath,1,RAT("\",cSemPath))+"TEMP"+cAppCode
cSemPath := cSemDir+"\"

  // nVersion is a filewide static.

IF nVersion = 1
   cIniFileW := GetWinDir()+"\QCSWIN.ini"
   cIniFile := CurDrive()+":\"+CurDir()+"\Qcswin.ini"
   cIniPath :=  "\" + CURDIR() + "\WIN\Qcswin.INI"
  ELSEIF nVersion = 2
   cIniFileW := GetWinDir()+"\QCSWIN.ini"
   cIniFile := CurDrive()+":\"+CurDir()+"\Qcswin.ini"
   cIniPath :=  "\" + CURDIR() + "\WIN\Qcswin.INI"
  ELSE
   cIniFileW := GetWinDir()+"\S360.ini"
   cIniFile := CurDrive()+":\"+CurDir()+"\S360.ini"
   cIniPath :=  "\" + CURDIR() + "\WIN\S360.INI"
ENDIF

   nRoutine := pp_tcode( nGet_code, nCe_num, nCompno, nSeed )

    DO CASE
        CASE nRoutine = 1                         // reset users limit to 1
            if pp_setvarnum(nLFhandle,VAR_LAN_LIMIT, 0) = 1 
               SysRefresh()
               CursorWait()
               cMessage := "User limit set to server only."
            endif
        CASE nRoutine = 6                         // clear workstation count
            if pp_setvarnum(nLFhandle,VAR_LAN_COUNT, 0) = 1 
               SysRefresh()
               CursorWait()
               cMessage := "Workstation counter cleared."
            endif
        CASE nRoutine = 7                         // authorize a new user
            if pp_countinc(nLFhandle,VAR_LAN_LIMIT) = 1  
               SysRefresh()
               CursorWait()
               cMessage := "Workstation limit increased by 1."
            endif
        CASE nRoutine = 8                         // authorize 5 more users
            if pp_getvarnum(nLFhandle,VAR_LAN_LIMIT,@nLimit) = 1 
               SysRefresh()
               CursorWait()
               nLimit := nLimit + 5
               if pp_setvarnum(nLFhandle,VAR_LAN_LIMIT,nLimit) = 1
                  SysRefresh()
                  CursorWait()
                  cMessage := "Workstation limit increased by 5."
               endif
            endif
        CASE nRoutine = 9                         // authorize 5 users
            nRet := pp_setvarnum(nLFhandle,VAR_LAN_LIMIT, 5)  

            SysRefresh()
            CursorWait()

*            LanLim2(7)   // ???  Don't want to set nNetname from server,
                          //      should only set it from workstation.
            if nRet = 1
                cMessage := "Workstation limit set to 5."
            else
                cMessage := "Error code: "+ALLTRIM(STR(nRet))
            endif
        CASE nRoutine = 11                         // revert to DEMO mode
            if pp_setvarnum(nLFhandle,VAR_LAN_LIMIT, 0) = 1 
               SysRefresh()
               CursorWait()
               if pp_setvarchar(nLFhandle,VAR_EXPIRE_TYPE, "E") = 1
                  SysRefresh()
                  CursorWait()
                  pp_setvarnum(nLFhandle,VAR_EXP_COUNT, 0)
                  SysRefresh()
                  CursorWait()
                  pp_copydelete(nLFhandle,COPYDEL_ENHANCED)
                  SysRefresh()
                  CursorWait()
                  pp_getvarnum(nLFhandle,VAR_EXP_COUNT,@nExpCount )
                  SysRefresh()
                  CursorWait()
                  cNewGama  := SPACE(5)
                  pp_encrypt(STR(nExpCount,2),cPwstr,@cNewGama)
                  cNewGama := SUBSTR(cNewGama,1,2)
                  IF nVersion = 3
                     cInitFil := RTRIM(cPath) + "\Wrk\InitFil2"
                    ELSEIF nVersion = 1
                     cInitFil := RTRIM(cPath) + "\Wrk\InitFile"
                    ELSEIF nVersion = 2
                     cInitFil := RTRIM(cPath) + "\Wrk\InitFile"
                    ELSE
                     cInitFil := RTRIM(cPath) + "\RESET\InitFile"
                  ENDIF
    
                  IF SELECT("InitFile") = 0
                   IF_NETOPEN (cInitFil) ALIAS InitFile SHARED
                     ELSE
                      Nonopen("Wrk\InitFile")
                      RETURN aBackLan
                   ENDIF
                  ENDIF
                  SELECT InitFile
                  IF InitFile->( Netlock( RECORDLOCK ) )
                     ELSE    // Netlock()
                       Nonlock("Wrk\InitFile")
                       RETURN aBackLan
                  ENDIF     // Netlock()

                  REPLACE Gama WITH cNewGama
                  REPLACE Testnum WITH 0

                  SysRefresh()
                  CursorWait()

                  InitFile->( dbCommit() )
                  InitFile->( dbUnLock() )

                  cMessage := "Program returned to single user DEMO mode."
               endif
            endif
        CASE nRoutine = 18                         // authorize 10 more users
            if pp_getvarnum(nLFhandle,VAR_LAN_LIMIT,@nLimit) = 1 //pp_lanplus( 10 ) = 1
               nLimit := nLimit + 10
               if pp_setvarnum(nLFhandle,VAR_LAN_LIMIT,nLimit) = 1
                  SysRefresh()
                  CursorWait()
                  cMessage := "Workstation limit increased by 10."
               endif
            endif
        CASE nRoutine = 0                         // error
               cMessage := "Incorrect code " + STR(nGet_code) + " entered. Please retry."
        Otherwise                               // anything else
               cMessage := "Incorrect code " + STR(nGet_code) + " entered. Please retry."
    ENDCASE

     IF pp_getvarnum(nLFhandle,VAR_LAN_LIMIT,@nLimit) = 0
           nLimit := 0
        ELSE
     ENDIF

     IF pp_getvarnum(nLFhandle,VAR_LAN_COUNT,@nOnline) = 0
           nOnline := 0
        ELSE
     ENDIF

     pp_getvarnum(nLFhandle,VAR_EXP_COUNT, @nCount)
       // On networks, we use cIniFile in Windows dir so each workstation
       // can have its own local InitFile. But single user installs use it 
       // in app dir.
    IF (nRoutine=7 .OR. nRoutine=8 .OR. nRoutine=9 .OR. nRoutine=18)

       LanLim2(nRoutine)

       // Create dir for semaphore files, if doesn't already exist...
       IF lIsDir(cSemDir) = .f.
          lMkDir(cSemDir)
       ENDIF

       IF !FILE(cIniFileW)
         COPY FILE (cIniFile) TO (cIniFileW)
       ENDIF

    ENDIF

    SysRefresh()
    CursorWait()

aBackLan    := ARRAY(3)
aBackLan[1] := nLimit
aBackLan[2] := cMessage
aBackLan[3] := nOnline

RETURN aBackLan
*----------------
 // Appath encryption and storage in auxiliary dbf file...
STATIC FUNCTION LanLim2(nCode)

LOCAL ;
      cEncrypt1 := SPACE(40) ,;
      cNetname  := ""        ,;
      cInitFil  := ""        ,;
      nNetname  := 0         ,;
      nSfcnt    := 0

IF nVersion = 3
   cInitFil := RTRIM(cPath) + "\Wrk\InitFil2"
  ELSEIF nVersion = 1
   cInitFil := RTRIM(cPath) + "\Wrk\InitFile"
  ELSEIF nVersion = 2
   cInitFil := RTRIM(cPath) + "\Wrk\InitFile"
  ELSE
   cInitFil := RTRIM(cPath) + "\RESET\InitFile"
ENDIF

IF SELECT("InitFile") = 0
 IF_NETOPEN (cInitFil) ALIAS InitFile SHARED
     // Check if InitFile has current fields; if not, add them.
*     nSfcnt := FCOUNT()
*     IF nSfcnt < 32     // Check if Wrk folder InitFile file has current layout.
*        SETCOPY(nSfcnt) // If not, then rebuild file with added fields
*     ENDIF
   ELSE
     Nonopen(IF(nVersion = 1 .OR. nVersion = 2,"Wrk\InitFile","Wrk\InitFil2"))
     RETURN aBack
 ENDIF
ENDIF
SELECT InitFile

nNetname := pp_compno( COMPNO_NETNAME , "" , cDrive  ) // legacy version of pp_compno()
cNetname := PADR(ALLTRIM(STR(nNetname)),35)


IF nCode = 7 .OR. nCode = 8 .OR. nCode = 9 .OR. nCode = 18
   pp_encrypt( cNetname, cPwstr3, @cEncrypt1 )
   IF InitFile->( Netlock( RECORDLOCK ) )
     ELSE    // Netlock()
       Nonlock("Wrk\InitFile")
       RETURN aBack
   ENDIF     // Netlock()
   REPLACE Appath WITH cEncrypt1
ENDIF
InitFile->( dbCommit() )
InitFile->( dbUnLock() )

RETURN cNetname
*lu-------------------
 // Check if another lan user can be added, and if so, then do it...
STATIC PROCEDURE Lanusers(n)

LOCAL ;
      cAppCode  := ""  ,;
      cLimit    := ""  ,;
      cUsers    := ""  ,;
      lFwrd     := .f. ,;
      lGoAhead  := .f. ,;
      nLimit    := 0   ,;
      nRet      := 0   ,;
      nUsers    := 0   ,;
      nUsersSem := 0   ,;
      oDlgD            ,;
      oLimit           ,;
      oUsers

CursorWait()

 // This NOT a PLUS file - it belongs to app...
cSemaphor := CurDrive() + ":\" +CurDir()+ "\Semaphor.ini"

/* Note: TWO methods of floating workstation authorization are used here,
         though only one is needed: 1) the old license file counter method and
                                    2) the new PLUS V4 semaphore files method.
  
         Both work, but the 2nd has these advantages:
             a) if the app crashes, users won't be blocked from getting in
                because the counter didn't get decremented.
             b) the use of semaphores in an inconspicuous parallel directory
                can make it harder for hackers to put copies of the app in
                multiple subdirs in order to get more workstation rights.
  
         But the 2nd method has one shortcoming: in networks with more than
         50 workstations, the time to create and lock a semaphore file gets
         longer, perhaps to an annoying degree. The first type doesn't consume
         more time to get on with a high number of workstations, so it is
         included here as an option. It can be commented out if you don't
         want to use it, though that's not necessary.
*/
cSemPath  := cDrive+":\"+CurDir()
cAppCode  := SUBSTR(cSemPath,RAT("\",cSemPath)+1,1)
cAppCode  := cAppCode + SUBSTR(cSemPath,LEN(cSemPath),1)
cSemPath  := SUBSTR(cSemPath,1,RAT("\",cSemPath))+"TEMP"+cAppCode+"\"
cSemPrefx := "pag"+SUBSTR(ALLTRIM(STR(nVersion)),LEN(ALLTRIM(STR(nVersion))),1)

nRet    := pp_getvarnum(nLFhandle, VAR_LAN_COUNT, @nUsers)
IF nRet = 1
   nRet := pp_semused(nLFhandle, 0, cSemPath, cSemPrefx, @nUsersSem)
ENDIF
IF nRet = 1
   IF nUsers > nUsersSem  // If there are less semaphores open than the number
                          // of VAR_LAN_USERS, then decrease that number to 
                          // match the number of open semaphores...
      pp_setvarnum(nLFhandle,VAR_LAN_COUNT,nUsersSem)

      SysRefresh()
      CursorWait()

   ENDIF                  // Just another way that the lan users count gets
ENDIF                     // corrected for user counts not deducted on abnormal
                          // exit.


// The first method...
IF pp_lancheck(nLFhandle) = 1   // can I add another user?

   lGoAhead := .t.    // yes, add another user.

   pp_countinc(nLFhandle, VAR_LAN_COUNT)

   SysRefresh()
   CursorWait()

/* The second method... (In this example, it's subsidiary to 1st method, but
                     not necessary to do it this way. Could use just 1 method.)

         Explanation of how we use the placement and naming of a semaphore 
         files directory to try to prevent unauthorized multiple copies on a 
         server of the same app from giving the user additional workstation 
         rights:

         Setup the path to semaphore files to a TEMPxx directory that is in a
         PARALLEL location on the disk to your app directory. As a parallel
         directory, it will likely be overlooked by someone who tries to get
         extra workstation rights by copying your app directory into multiple
         subdirectories on their server's hard drive.  Thus, when they try to
         run the bogus app copies, the TEMPxx dir won't be found and they will
         be locked out of adding more workstations illegally. This isn't super
         secure, but it will discourage casual attempts to hack more network
         user rights. (Note: "TEMPxx" is used to name the semaphore dir in hopes
         that a hacker would overlook it as unimportant - especially because
         if they look in there when the app isn't running, it is empty.)
         HOWEVER, when you install your app on a customer's server, be sure to
         also install the parallel TEMPxx dir - if you forget this, then your 
         app won't work from network workstations.

         Example of PARALLEL directories TEMPxx and MyApp:
            p:\TEMPxx p:\MyApp  OR:  p:\AnyDir\TEMPxx p:\AnyDir\MyApp

         The "xx" part of TEMPxx represents cAppCode, comprised of the first
         and last characters of the app directory name.  This way, if a hacker
         tries to duplicate your app dir within the same disk subdir with a
         different name for each bogus app dir (in order to have multiple
         copies of the same app within the same subdir), then the original
         TEMPxx dir won't be accessible from the bogus app dir's because the
         cAppCode derived from their dir names will be different than the legal
         original app dir. In this way you're protected if the hacker tries to
         copy the app dir either into multiple subdirs on the same server or 
         into the same subdir by different names.

         Now for the second method, where...

      cSemPath := cDrive+":\"+CurDir()
      cAppCode := SUBSTR(cSemPath,RAT("\",cSemPath)+1,1)
      cAppCode := cAppCode + SUBSTR(cSemPath,LEN(cSemPath),1)
      cSemPath := SUBSTR(cSemPath,1,RAT("\",cSemPath))+"TEMP"+cAppCode+"\"

      cSemPrefx is designed to enable multiple DIFFERENT exe's (each for a
      different app) within the same app dir to access the same semaphores 
      dir without interfering with each other, because each exe's semaphores
      gets a different name prefix based on the app's nVersion code here. 

      cSemPrefx := "pag"+SUBSTR(ALLTRIM(STR(nVersion)),LEN(ALLTRIM(STR(nVersion))),1)

      (Note: both cSemPath and cSemPrefx were loaded earlier in this proc.)
*/
      // Now that we've determined we can run on this computer,
      // see if we can open a semaphore file for this application...
      IF pp_semopen(nLFhandle, 0, cSemPath, cSemPrefx, @nSemhandle) == PP_SUCCESS

         SysRefresh()
         CursorWait()

         lGoAhead := .t.

         // Note: semaphore files are temporary files created by pp_semopen()
         //       and deleted by pp_semclose().
         aBack[5] := nSemhandle  // we need to send this to the app so it can
                                 // close the semaphore upon the user's exit.

         // Everything checked out, okay to launch app.
         lGoAhead := .t.

         // Optional info on # of user rights available, for testing purposes...
*        pp_semcount(nLFhandle,0, cSemPath, cSemPrefx, @nUsers)

*        msginfo(nUsers,"nUsers")

/*          This func must be inserted into the app's EXIT PROCEDURE to close
            the user's semaphore upon exiting the app. It should also be
            performed in the error handler, which should grab the nSemhandle
            from the app's main prg (where it should be a file wide static)... 

            pp_semclose(nSemhandle) 
*/

      ELSE

         lGoAhead := .f. 

         LanUsers2(1)

         SysRefresh()
         CursorWait()

      ENDIF  // pp_semopen()
 ELSE

      lGoAhead := .f.

      LanUsers2(2)

      SysRefresh()
      CursorWait()

ENDIF // pp_lancheck() = 1


RETURN
*lu2----------------
  // Message screen - LAN users limit has been reached...
STATIC PROCEDURE LanUsers2(n)

LOCAL ;
      cLimit    := ""  ,;
      cUsers    := ""  ,;
      lFwrd     := .f. ,;
      nLimit    := 0   ,;
      nRet      := 0   ,;
      nUsers    := 0   ,;
      oDlgD,         ;
      oLimit,        ;
      oUsers

    // This NOT a PLUS file - it belongs to app...
   cSemaphor := CurDrive() + ":\" +CurDir()+ "\Semaphor.ini"
   
*  MsgInfo(n,"Called-from #")  // n tells us where this proc was called from.

   nRet := pp_getvarnum(nLFhandle, VAR_LAN_LIMIT, @nLimit)
   IF nRet > 0
      nRet := pp_getvarnum(nLFhandle, VAR_LAN_COUNT, @nUsers)
   ELSE
      MsgAlert("License File lan data not available.")
      RETURN
   ENDIF
   IF nRet > 0
      cLimit := LTRIM(STR(nLimit))
      cUsers := LTRIM(STR(nUsers)) 
   ELSE
      MsgAlert("License File lan data not available.")
      RETURN
   ENDIF

   MsgBeep()

   DEFINE DIALOG oDlgD RESOURCE "STARTUP_D" OF oWnd

   // "Allowed simultaneous users limit has been reached for this program."

   // "Simultaneous users allowed:"
   REDEFINE SAY oLimit ID 21 PROMPT cLimit OF oDlgD

   // "Current users:"
   REDEFINE SAY oUsers ID 22 PROMPT cUsers OF oDlgD

   // Reset if user count wrong due to crash...    
   REDEFINE BUTTON ID 1 OF oDlgD ; // Only works if just 1 user signed on.  
     ACTION ( lFwrd := .t. , oDlgD:End(), ClearUsers() )

   REDEFINE BUTTON ID 2 OF oDlgD ;   // Cancel
     ACTION ( lFwrd := .f., oDlgD:End() )  

   REDEFINE BUTTON ID 3 OF oDlgD ;   // Tech
     ACTION ( lFwrd := .t. , oDlgD:End(), ;
              Get_Lanc() ;
             )

   REDEFINE BUTTON ID 4 OF oDlgD ;   // Help
     ACTION ( HelpTopic(7018) )

   ACTIVATE DIALOG oDlgD CENTERED ;
      ON INIT (oDlgD:Move( nWtop+37, nWleft+50 ), SysWait(.2), SetFocus(oDlgD)) RESIZE16


  IF lFwrd = .f.
     WritePProString("STOPS","Stopq" , "F", cSemaphor )     
     lExit := .t.
     aBack[8]  := nExTrue
  ENDIF


RETURN
*----------------
  // Clear counter of lan users count if an app crash left the count high...
STATIC PROCEDURE ClearUsers()

LOCAL cUser := "User"

IF SELECT("User") = 0
  // USER.DBF is a dummy file used to make sure that reset of users count
  // can only be done when everyone else is signed off.

   IF nVersion = 3
      cUser := "User360"
     ELSE
      cUser := "User"
   ENDIF
   IF_NETOPEN (cUser) ALIAS User EXCLUSIVE
        if pp_setvarnum(nLFhandle, VAR_LAN_COUNT, 0 ) = 1
           MsgInfo("Workstation counter cleared.")
        endif
      ELSE
*       MsgAlert("All users must sign off before reset.")
        // Note: this same message in Netopen.prg
        lExit := .t.
        aBack[8]  := nExTrue
        EXIT()
   ENDIF
 ELSE
  MsgOk("Can't reset now. Try again later.")
ENDIF

RETURN
*----------------
  // "Stopq" processing in Semaphor.ini for Lan users...
STATIC PROCEDURE Leadin(cLFile)

LOCAL ;
      aRay      := {}  ,;
      cIniFile  := ""  ,;
      cIniFileW := ""  ,;
      cStopq    := "F" ,;
      lFwrd     := .t. ,;
      nRet      := 0   ,;
      nUsers    := 0   ,;
      oDlgE

IF nVersion = 1
   cIniFileW := GetWinDir()+"\QCSWIN.ini"
   cIniFile := CurDrive()+":\"+CurDir()+"\Qcswin.ini"
  ELSEIF nVersion = 2
   cIniFileW := GetWinDir()+"\QCSWIN.ini"
   cIniFile := CurDrive()+":\"+CurDir()+"\Qcswin.ini"
  ELSE
   cIniFileW := GetWinDir()+"\S360.ini"
   cIniFile := CurDrive()+":\"+CurDir()+"\S360.ini"
ENDIF

cSemaphor := CurDrive() + ":\" +CurDir()+ "\Semaphor.ini"
IF !FILE(cSemaphor)
   MEMOWRIT(cSemaphor, "[STOPS]"+CRLF+"Stopq=F"+CRLF+"Stopmem=F"+CRLF+"Stopjpw=F"+CRLF )
ENDIF

aRay   := Directory(rtrim(cPath) + "\"+cLFile+".lf") // Won't work! LF date doesn't change.

dTdate  := aRay[1,3]

nRet := pp_getvarnum(nLFhandle,VAR_LAN_COUNT,@nUsers)

cStopq :=  GetPvProfString("STOPS","Stopq" , "F", cSemaphor )     

// IF dTdate < DATE()  // If this is first startup of the day...
//                           \/
IF nRet = 1 .AND. nUsers < 1 .OR. dTdate < DATE()
   IF cStopq == "T"   // <- Left-over from a prior crash, if nobody else online
      WritePProString("STOPS","Stopq" , "F", cSemaphor )     
   ENDIF
ENDIF

// Check if nobody else online...  
DO WHILE GetPvProfString( "STOPS" , "Stopq"    , "F"  , cSemaphor ) == "T" 
   // If someone else has started their entry into the app right now,
   // delay going in until other user is through the entry gate:

   MsgBeep()

   DEFINE DIALOG oDlgE RESOURCE "STARTUP_E" OF oWnd

   // "SHORT DELAY - while startup processing is done at another workstation."

   REDEFINE BUTTON ID 1 OF oDlgE ;   // Retry
     ACTION ( lFwrd := .t. , oDlgE:End() )

   REDEFINE BUTTON ID 2 OF oDlgE ;   // Cancel
     ACTION ( lFwrd := .f., oDlgE:End() )  

   ACTIVATE DIALOG oDlgE CENTERED ;
      ON INIT (oDlgE:Move( nWtop+37, nWleft+50 ), SysWait(.2), SetFocus(oDlgE)) RESIZE16

            IF lFwrd = .f.     // User not allowed to go forward
               WritePProString("STOPS","Stopq" , "F", cSemaphor )     
               lExit := .t.    // Exit() will be forced on return to main app,
               aBack[8] := nExTrue
               EXIT            // so elim exit message.
            ENDIF

   SysRefresh()
ENDDO

IF lFwrd = .t.
   // Create temp semaphore setting to prevent other users from entering app
   // until this user is finished going through the entry gate:
   WritePProString("STOPS","Stopq" , "T", cSemaphor )     
ENDIF

RETURN
*----------------------
  // Special modal dialog to prevent user from avoiding Pro PLUS constraints...
STATIC FUNCTION MsgTF(cMsg1,cMsg2,cMsg3)

LOCAL ;
      lAns    := .f. ,;
      oDlgTF         ,;
      oMsg1          ,;
      oMsg2          ,;
      oMsg3

 DEFINE DIALOG oDlgTF RESOURCE "MsgTF" OF oWnd

 REDEFINE SAY oMsg1 PROMPT cMsg1 ID 21 OF oDlgTF
 REDEFINE SAY oMsg2 PROMPT cMsg2 ID 22 OF oDlgTF
 REDEFINE SAY oMsg3 PROMPT cMsg3 ID 23 OF oDlgTF

 REDEFINE BUTTON ID 11 OF oDlgTF ;   // Yes
   ACTION ( lAns := .t., oDlgTF:End() )

 REDEFINE BUTTON ID 2 OF oDlgTF ;   // Cancel
   ACTION ( lAns := .f., oDlgTF:End )  

 ACTIVATE DIALOG oDlgTF CENTERED ;
   ON INIT (SetFocus(oDlgTF)) RESIZE16

RETURN lAns
*----------------------
  // Special modal dialog to prevent user from avoiding Pro PLUS constraints...
STATIC PROCEDURE MsgOK(cMsg12,cMsg13)

 LOCAL oDlgOK,oMsg12,oMsg13

 DEFINE DIALOG oDlgOK RESOURCE "MsgOK" of oWnd
 REDEFINE SAY oMsg12 PROMPT cMsg12 ID 22 OF oDlgOK
 REDEFINE SAY oMsg13 PROMPT cMsg13 ID 23 OF oDlgOK
 REDEFINE BUTTON ID 11 OF oDlgOK ;   // Yes
     ACTION ( oDlgOK:End() )
 ACTIVATE DIALOG oDlgOK CENTERED ;
     ON INIT (SetFocus(oDlgOK)) RESIZE16

*----------------------
  // Rebuild an index so its date can be used as an auxiliary demo
  // enforcement tool...  (Nonworking sample - use an actual file of your own.)
STATIC Procedure BldMyfileIdx()

LOCAL ;
      cUsers  := ""                     ,;
      MyFil   := "MyFile"               ,;
      MyfileX := "MyFile" + IndexExt()  ,;
      nUsers  := 0

FIELD Field1,Field2

pp_getvarnum(nLFhandle,VAR_LAN_COUNT,@nUsers)

IF nUsers <= 1   // Only rebuild index if this is only user online.

  IF FILE((MyFileX))
     ERASE (MyFileX)
  ENDIF
  IF_NETOPEN (Myfil) ALIAS Myfile EXCLUSIVE
     
    IF !FILE((MyfileX))
      INDEX ON Field1 TAG Field1
      INDEX ON Field2 TAG Field2
    ENDIF
      dbcloseall()
   ELSE
      Nonopen("MyFile")
  ENDIF

ENDIF

RETURN

*--------------------------------------------
  // XBase++ requirement
PROCEDURE AppSys  

RETURN

//---------------------------------------------------------------------------//
/*
// Auxiliary file fields to include:
*InitFile.dbf
STATIC FUNCTION Bldfile(cFile)
LOCAL  C_fld[0]
  Aadd(C_fld, { "SRECNO"          ,"N", 2,0})
  Aadd(C_fld, { "TESTNUM"         ,"N",12,0})
  Aadd(C_fld, { "GAMA"            ,"C", 2,0})
  Aadd(C_fld, { "STAND"           ,"C", 8,0})
  Aadd(C_fld, { "DISKPRO"         ,"C",12,0})
  Aadd(C_fld, { "TENFLD"          ,"C",10,0})
  Aadd(C_fld, { "APPATH"          ,"C",40,0})
  Aadd(C_fld, { "NETNAME"         ,"C",40,0})

Dbcreate(RTRIM(cPath) + "\Wrk\" + cFile + ".DBF",C_fld  )

RETURN nil

*/